@@ -4,7 +4,7 @@
  *
  *   Purpose:	Implements class interface for netCDF over C interface
  *
- *   $Header: /upc/share/CVS/netcdf-3/cxx/netcdf.cpp,v 1.17 2008/05/05 20:31:20 ed Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/cxx/netcdf.cpp,v 1.18 2009/03/10 15:20:54 russ Exp $
  *********************************************************************/
 
 #include <config.h>
@@ -355,7 +355,7 @@ int NcFile::id( void ) const
 }
 
 NcFile::NcFile( const char* path, FileMode fmode, 
-		size_t* chunksizeptr, size_t initialsize, FileFormat fformat  )
+		size_t* bufrsizeptr, size_t initialsize, FileFormat fformat  )
 {
     NcError err(NcError::silent_nonfatal); // constructor must not fail
 
@@ -380,7 +380,7 @@ NcFile::NcFile( const char* path, FileMode fmode,
     case ReadOnly:
 	// use netcdf-3 interface to permit specifying tuning parameter
 	status = NcError::set_err(
-				  nc__open(path, mode, chunksizeptr, &the_id)
+				  nc__open(path, mode, bufrsizeptr, &the_id)
 				  );
 	if(status != NC_NOERR)
 	{
@@ -396,7 +396,7 @@ NcFile::NcFile( const char* path, FileMode fmode,
 	// use netcdf-3 interface to permit specifying tuning parameters
 	status = NcError::set_err(
 				  nc__create(path, mode, initialsize,
-				      chunksizeptr, &the_id)
+				      bufrsizeptr, &the_id)
 				  );
 	if(status != NC_NOERR)
 	{
