@@ -44,7 +44,7 @@
   function nf90_inq_grps(ncid, numgrps, ncids)
     integer, intent(in) :: ncid
     integer, intent(out) :: numgrps
-    integer, intent(out) :: ncids
+    integer, dimension(:), intent(out) :: ncids
     integer :: nf90_inq_grps
   
     nf90_inq_grps = nf_inq_grps(ncid, numgrps, ncids)
@@ -67,6 +67,15 @@
     nf90_inq_grp_ncid = nf_inq_grp_ncid(ncid, name, grpid)
   end function nf90_inq_grp_ncid
   ! -----------
+  function nf90_inq_grp_full_ncid(ncid, full_name, grpid)
+    integer, intent(in) :: ncid
+    character (len = *), intent(in) :: full_name
+    integer, intent(out) :: grpid
+    integer :: nf90_inq_grp_full_ncid
+  
+    nf90_inq_grp_full_ncid = nf_inq_grp_full_ncid(ncid, full_name, grpid)
+  end function nf90_inq_grp_full_ncid
+  ! -----------
   function nf90_inq_grp_parent(ncid, parent_ncid)
     integer, intent(in) :: ncid
     integer, intent(out) :: parent_ncid
@@ -95,7 +104,7 @@
   function nf90_inq_varids(ncid, nvars, varids)
     integer, intent(in) :: ncid
     integer, intent(out) :: nvars
-    integer, intent(out) :: varids
+    integer, dimension(:), intent(out) :: varids
     integer :: nf90_inq_varids
   
     nf90_inq_varids = nf_inq_varids(ncid, nvars, varids)
@@ -104,7 +113,7 @@
   function nf90_inq_dimids(ncid, ndims, dimids, include_parents)
     integer, intent(in) :: ncid
     integer, intent(out) :: ndims
-    integer, intent(out) :: dimids
+    integer, dimension(:), intent(out) :: dimids
     integer, intent(out) :: include_parents
     integer :: nf90_inq_dimids
   
@@ -113,13 +122,24 @@
   ! -----------
   function nf90_inq_typeids(ncid, ntypes, typeids)
     integer, intent(in) :: ncid
-    integer, intent(out) :: ntypes
-    integer, intent(out) :: typeids
+    integer, optional, intent(out) :: ntypes
+    integer, dimension(:), optional, intent(out) :: typeids
     integer :: nf90_inq_typeids
-  
+    
     nf90_inq_typeids = nf_inq_typeids(ncid, ntypes, typeids)
+
   end function nf90_inq_typeids
   ! -----------
+  function nf90_inq_typeid(ncid, name, typeid)
+    integer, intent(in) :: ncid
+    character (len = *), intent(in) :: name
+    integer, optional, intent(out) :: typeid
+    integer :: nf90_inq_typeid
+    
+    nf90_inq_typeid = nf_inq_typeid(ncid, name, typeid)
+
+  end function nf90_inq_typeid
+  ! -----------
   function nf90_def_grp(parent_ncid, name, new_ncid)
     integer, intent(in) :: parent_ncid
     character (len = *), intent(in) :: name
@@ -548,9 +568,9 @@
      integer, dimension(:), optional, intent( in) :: start, count, stride, map
      integer                                      :: nf90_get_var_any
  
-     integer, dimension(nf90_max_var_dims) :: localIndex, textDimIDs
+     integer, dimension(nf90_max_var_dims) :: textDimIDs
      integer, dimension(nf90_max_var_dims) :: localStart, localCount, localStride 
-     integer                               :: counter, stringLength
+     integer                               :: stringLength
  
      ! Set local arguments to default values
      localStart (:)  = 1
