@@ -30,7 +30,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-/* "$Id: netcdf.h,v 2.122 2009/03/25 01:48:32 dmh Exp $" */
+/* "$Id: netcdf.h,v 2.130 2010/01/12 15:44:57 ed Exp $" */
 
 #ifndef _NETCDF_
 #define _NETCDF_
@@ -53,19 +53,6 @@ typedef enum {
 	NC_INT =	4,	/* signed 4 byte integer */
 	NC_FLOAT =	5,	/* single precision floating point number */
 	NC_DOUBLE =	6	/* double precision floating point number */
-#ifndef USE_NETCDF4
-#ifdef USE_DAP
-	,
-	NC_URL = 33,
-	NC_SET = 34,
-	NC_Dataset = 35,
-	NC_Sequence = 36,
-	NC_Structure = 37,
-	NC_Grid = 38,
-	NC_Dimension = 39,
-	NC_Primitive = 40
-#endif
-#endif
 } nc_type;
 
 
@@ -210,9 +197,16 @@ typedef enum {
 #define NC_EAXISTYPE    (-65)   /* Unknown axis type. */
 
 /* Following errors are added for DAP */
-#define NC_EDAP         (-66)   /* Generic DAP error */
+#define NC_EDAP         (-66)   /* Generic DAP client error */
 #define NC_ECURL        (-67)   /* Generic libcurl error */
 #define NC_EIO          (-68)   /* Generic IO error */
+#define NC_ENODATA      (-69)   /* Attempt to access variable with no data */
+#define NC_EDAPSVC      (-70)   /* DAP Server side error */
+#define NC_EDAS		(-71)   /* Malformed or inaccessible DAS */
+#define NC_EDDS		(-72)   /* Malformed or inaccessible DDS */
+#define NC_EDATADDS	(-73)   /* Malformed or inaccessible DATADDS */
+#define NC_EDAPURL	(-74)   /* Malformed DAP URL */
+#define NC_EDAPCONSTRAINT (-75)   /* Malformed DAP Constraint*/
 
 /*
  * The Interface
@@ -247,6 +241,10 @@ MSC_EXTRA int ncopts;
 EXTERNL const char *
 nc_inq_libvers(void);
 
+/* Find name and size of a type. */
+EXTERNL int
+nc_inq_type(int ncid, nc_type xtype, char *name, size_t *size);
+
 EXTERNL const char *
 nc_strerror(int ncerr);
 
