@@ -2,7 +2,7 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: string.c,v 1.74 2009/02/20 20:35:03 dmh Exp $ */
+/* $Id: string.c,v 1.75 2009/09/28 19:14:36 dmh Exp $ */
 
 #include "nc.h"
 #include "rename.h"
@@ -282,3 +282,22 @@ set_NC_string(NC_string *ncstrp, const char *str)
 	return NC_NOERR;
 }
 
+/**************************************************/
+/* Provide local alternatives for unix functions
+   not available on all machines. Place here so that
+   all subsequence code modules can use it.
+*/
+
+#ifndef HAVE_STRDUP
+char*
+nulldup(const char* s)
+{
+    char* dup;
+    if(s == NULL) return NULL;
+    dup = malloc(strlen(s)+1);
+    strcpy(dup,s);
+    return dup;
+}
+#endif
+
+/**************************************************/
