@@ -11,10 +11,15 @@ Copyright 2003-2005, University Corporation for Atmospheric
 Research. See COPYRIGHT file for copying and redistribution
 conditions.
 
-$Id: nc4attr.c,v 1.52 2008/06/10 15:38:34 ed Exp $
+$Id: nc4attr.c,v 1.72 2009/10/18 10:42:57 ed Exp $
 */
 
 #include <nc4internal.h>
+#include "rename4.h"
+
+#ifdef USE_PNETCDF
+#include <pnetcdf.h>
+#endif
 
 int nc4typelen(nc_type type);
 
@@ -98,7 +103,7 @@ nc4_get_att(int ncid, NC_FILE_INFO_T *nc, int varid, const char *name,
 	 (att->xtype == NC_UBYTE || att->xtype == NC_BYTE)))
    {
       need_to_convert++;
-      if (!(bufr = nc_malloc((size_t)(att->len * type_size))))
+      if (!(bufr = malloc((size_t)(att->len * type_size))))
 	 return NC_ENOMEM;
       if ((retval = nc4_convert_type(att->data, bufr, att->xtype, 
 				     mem_type, (size_t)att->len, &range_error, 
@@ -134,7 +139,7 @@ nc4_get_att(int ncid, NC_FILE_INFO_T *nc, int varid, const char *name,
 	 for (i = 0; i < att->len; i++)
 	 {
 	    vldest[i].len = att->vldata[i].len;
-	    if (!(vldest[i].p = nc_malloc(vldest[i].len * base_typelen)))
+	    if (!(vldest[i].p = malloc(vldest[i].len * base_typelen)))
 	       BAIL(NC_ENOMEM);
 	    memcpy(vldest[i].p, att->vldata[i].p, vldest[i].len * base_typelen);
 	 }
@@ -166,7 +171,7 @@ nc4_get_att(int ncid, NC_FILE_INFO_T *nc, int varid, const char *name,
    }
 
  exit:
-   if (need_to_convert) nc_free(bufr);
+   if (need_to_convert) free(bufr);
    if (retval)
       return retval;
    if (range_error)
@@ -277,6 +282,11 @@ nc4_put_att(int ncid, NC_FILE_INFO_T *nc, int varid, const char *name,
 	file_type == NC_STRING || mem_type == NC_STRING))
       return NC_ECHAR;
 
+   /* For classic mode file, only allow atts with classic types to be
+    * created. */
+   if (h5->cmode & NC_CLASSIC_MODEL && file_type > NC_DOUBLE)
+      return NC_ESTRICTNC3;
+
    /* Add to the end of the attribute list, if this att doesn't
       already exist. */
    if (new_att)
@@ -339,10 +349,12 @@ nc4_put_att(int ncid, NC_FILE_INFO_T *nc, int varid, const char *name,
       /* Allocate space for the fill value. */
       if (type_info && type_info->class == NC_VLEN)
 	 size = sizeof(hvl_t);
+      else if (var->xtype == NC_STRING)
+	 size = strlen(*(char **)data) + 1;
       else
 	 size = type_size;
 
-      if (!(var->fill_value = nc_malloc(size)))
+      if (!(var->fill_value = malloc(size)))
 	 return NC_ENOMEM;
 
       /* Copy the fill_value. */
@@ -355,10 +367,12 @@ nc4_put_att(int ncid, NC_FILE_INFO_T *nc, int varid, const char *name,
 	    return NC_ENOMEM;
 	 memcpy(fv_vlen->p, in_vlen->p, in_vlen->len * size);
       }
+      else if (var->xtype == NC_STRING)
+	 strcpy((char *)(var->fill_value), *(char **)data);
       else
 	 memcpy(var->fill_value, data, type_size);
 
-      /* Mark the var and all it's atts as dirty, so they get
+      /* Mark the var and all its atts as dirty, so they get
        * rewritten. */
       var->dirty++;
       for (varatt = var->att; varatt; varatt = varatt->next)
@@ -366,34 +380,29 @@ nc4_put_att(int ncid, NC_FILE_INFO_T *nc, int varid, const char *name,
    }
 
    /* Copy the attribute data, if there is any. VLENs and string
-    * arrays have to be handled specially. If this is a fi*/
-   if (type && type->class == NC_VLEN && data)
+    * arrays have to be handled specially. */
+   if (type && type->class == NC_VLEN && data && att->len)
    {
       const hvl_t *vldata1;
 
       vldata1 = data;
-      if (!(att->vldata = nc_malloc(att->len * sizeof(hvl_t))))
+      if (!(att->vldata = malloc(att->len * sizeof(hvl_t))))
 	 BAIL(NC_ENOMEM);	 
       for (i = 0; i < att->len; i++)
       {
 	 att->vldata[i].len = vldata1[i].len;
-	 if (!(att->vldata[i].p = nc_malloc(type_size * att->vldata[i].len)))
+	 if (!(att->vldata[i].p = malloc(type_size * att->vldata[i].len)))
 	    BAIL(NC_ENOMEM);
 	 memcpy(att->vldata[i].p, vldata1[i].p, type_size * att->vldata[i].len);
       }
    }
-   else if (file_type == NC_STRING && data)
+   else if (file_type == NC_STRING && data && att->len)
    {
       LOG((4, "copying array of NC_STRING"));
-      if (!(att->stdata = nc_malloc(sizeof(char *) * att->len)))
+      if (!(att->stdata = malloc(sizeof(char *) * att->len)))
 	 BAIL(NC_ENOMEM);	 
       for (i = 0; i < att->len; i++)
       {
-      /* Here I use malloc, not nc_malloc, because although I am in
-       * charge of allocating this space with atts that the user puts,
-       * the HDF5 library allocates it when the att is read. Since I
-       * just have one attribute freeing routine, I can't use
-       * nc_malloc here, because I can't get HDF5 to use nc_malloc. */
 	 LOG((5, "copying string %d of size %d", i, strlen(((char **)data)[i]) + 1));
 	 if (!(att->stdata[i] = malloc(strlen(((char **)data)[i]) + 1)))
 	    BAIL(NC_ENOMEM);
@@ -406,8 +415,8 @@ nc4_put_att(int ncid, NC_FILE_INFO_T *nc, int varid, const char *name,
       if (att->len)
       {
 	 if (!new_att)
-	    nc_free (att->data);
-	 if (!(att->data = nc_malloc(att->len * type_size)))
+	    free (att->data);
+	 if (!(att->data = malloc(att->len * type_size)))
 	    BAIL(NC_ENOMEM);
 	 if (type)
 	 {
@@ -452,6 +461,20 @@ nc_inq_att(int ncid, int varid, const char *name, nc_type *xtypep, size_t *lenp)
    if (!(nc = nc4_find_nc_file(ncid)))
       return NC_EBADID;
 
+#ifdef USE_PNETCDF
+   /* Take care of files created/opened with parallel-netcdf library. */
+   if (nc->pnetcdf_file)
+   {
+      MPI_Offset mpi_len;
+      int ret;
+
+      if ((ret = ncmpi_inq_att(nc->int_ncid, varid, name, xtypep, &mpi_len)))
+	 return ret;
+      if (lenp)
+	 *lenp = mpi_len;
+   }
+#endif /* USE_PNETCDF */
+
    /* Handle netcdf-3 files. */
    if (!nc->nc4_info)
       return nc3_inq_att(nc->int_ncid, varid, name, xtypep, lenp);
@@ -472,6 +495,12 @@ nc_inq_attid(int ncid, int varid, const char *name, int *attnump)
    if (!(nc = nc4_find_nc_file(ncid)))
       return NC_EBADID;
 
+#ifdef USE_PNETCDF
+   /* Take care of files created/opened with parallel-netcdf library. */
+   if (nc->pnetcdf_file)
+      return ncmpi_inq_attid(nc->int_ncid, varid, name, attnump);
+#endif /* USE_PNETCDF */
+
    /* Handle netcdf-3 files. */
    if (!nc->nc4_info)
       return nc3_inq_attid(nc->int_ncid, varid, name, attnump);
@@ -511,6 +540,12 @@ nc_inq_attname(int ncid, int varid, int attnum, char *name)
    if (!(nc = nc4_find_nc_file(ncid)))
       return NC_EBADID;
 
+#ifdef USE_PNETCDF
+   /* Take care of files created/opened with parallel-netcdf library. */
+   if (nc->pnetcdf_file)
+      return ncmpi_inq_attname(nc->int_ncid, varid, attnum, name);
+#endif /* USE_PNETCDF */
+
    /* Handle netcdf-3 files. */
    if (!nc->nc4_info)
       return nc3_inq_attname(nc->int_ncid, varid, attnum, name);
@@ -526,6 +561,202 @@ nc_inq_attname(int ncid, int varid, int attnum, char *name)
    return NC_NOERR;
 }
 
+/* Compare two netcdf types for equality. Must have the ncids as well,
+ * to find nested types. */
+static int
+compare_nc_types(int ncid1, NC_TYPE_INFO_T *t1, int ncid2, NC_TYPE_INFO_T *t2, int *equal)
+{
+   NC_FIELD_INFO_T *f1, *f2;
+   int equal1, ret;
+
+   assert(t1 && t2 && equal);
+
+   /* Assume the types are not equal. If we find any inequality, then
+    * exit with NC_NOERR and we're done. */
+   *equal = 0;
+
+   /* Atomic types are so easy! */
+   if (t1->nc_typeid <= NC_STRING)
+   {
+      if (t2->nc_typeid != t1->nc_typeid)
+	 return NC_NOERR;
+   }
+   else 
+   {
+      NC_HDF5_FILE_INFO_T *h5_1, *h5_2;
+      NC_GRP_INFO_T *grp1, *grp2;
+      NC_ENUM_MEMBER_INFO_T *m1, *m2;
+      NC_TYPE_INFO_T *ft1, *ft2;
+
+      /* Check the obvious. */
+      if (t1->size != t2->size || t1->class != t2->class || strcmp(t1->name, t2->name))
+	 return NC_NOERR;
+
+      /* Learn some stuff for further comarisons. */
+      if ((ret = nc4_find_grp_h5(ncid1, &grp1, &h5_1)))
+	 return ret;
+      if ((ret = nc4_find_grp_h5(ncid2, &grp2, &h5_2)))
+	 return ret;
+
+      /* Check user-defined types in detail. */
+      switch(t1->class)
+      {
+	 case NC_VLEN:
+	    if (t1->base_nc_type <= NC_STRING)
+	    {
+	       if (t1->base_nc_type != t2->base_nc_type)
+		  return NC_NOERR;
+	    }
+	    else
+	    {
+	       /* User defined type in VLEN! */
+	       if ((ret = nc4_find_type(h5_1, t1->base_nc_type, &ft1)))
+		  return ret;
+	       if ((ret = nc4_find_type(h5_2, t2->base_nc_type, &ft2)))
+		  return ret;
+	       if ((ret = compare_nc_types(ncid1, ft1, ncid2, ft2, &equal1)))
+		  return ret;
+	       if (!equal1)
+		  return NC_NOERR;
+	    }
+	    break;
+	 case NC_OPAQUE:
+	    /* Alread checked size above. */
+	    break;
+	 case NC_ENUM:
+	    if (t1->base_nc_type != t2->base_nc_type || t1->num_enum_members != 
+		t2->num_enum_members)
+	       return NC_NOERR;
+	    for (m1 = t1->enum_member, m2 = t2->enum_member; m1; m1 = m1->next, m2 = m2->next)
+	       if (strcmp(m1->name, m2->name) || memcmp(m1->value, m2->value, t1->size))
+		  return NC_NOERR;
+	    break;
+	 case NC_COMPOUND:
+	    if (t1->num_fields != t2->num_fields)
+	       return NC_NOERR;
+	    /* Compare each field. Each must be equal! */
+	    for (f1 = t1->field, f2 = t2->field; f1; f1 = f1->next, f2 = f2->next)
+	    {
+	       /* Handle atomic types. */
+	       if (f1->nctype <= NC_STRING)
+	       {
+		  if (f1->nctype != f2->nctype)
+		     return NC_NOERR;
+		  else
+		     continue;
+	       }
+
+	       /* Dang! *More* user-defined types! Look up the field
+		* types in each file. */
+	       if ((ret = nc4_find_type(h5_1, f1->nctype, &ft1)))
+		  return ret;
+	       if ((ret = nc4_find_type(h5_2, f2->nctype, &ft2)))
+		  return ret;
+	       
+	       /* Compare user-defined field types. */
+	       if ((ret = compare_nc_types(ncid1, ft1, ncid2, ft2, &equal1)))
+		  return ret;
+	       if (!equal1)
+		  return NC_NOERR;
+	    }
+	    break;
+	 default:
+	    return NC_EINVAL;
+      }
+   }
+   *equal = 1;
+   return NC_NOERR;
+}
+/* Find a type equal to type1 (in ncid1) starting search in grp2 (in
+ * ncid2) and point type2 at the resulting type (in ncid2). */
+static int 
+rec_find_nc_type(int ncid1, NC_TYPE_INFO_T *type1, int ncid2, 
+		 NC_GRP_INFO_T *grp2, NC_TYPE_INFO_T **type2)
+{
+   NC_GRP_INFO_T *g;
+   NC_TYPE_INFO_T *t;
+   int ret, equal = 0;
+
+   assert(grp2 && type1 && type2);
+   
+   /* Assume we can't find it. */
+   *type2 = NULL;
+
+   /* Type in this group? */
+   for (t = grp2->type; t; t = t->next)
+      if (!strcmp(t->name, type1->name))
+      {
+	 if ((ret = compare_nc_types(ncid1, type1, ncid2, t, &equal)))
+	    return ret;
+	 if (equal)
+	 {
+	    *type2 = t;
+	    return NC_NOERR;
+	 }
+      }
+
+   /* Still not found? Search subgroups, calling self recursively. */
+   if (!t && grp2->children)
+      for (g = grp2->children; g; g = g->next)
+      {
+	 ret = rec_find_nc_type(ncid1, type1, ncid2, g, &t);
+	 if (ret && ret != NC_EBADTYPE)
+	    return ret;
+
+	 if (t)
+	 {
+	    *type2 = t;
+	    return NC_NOERR;
+	 }
+      }
+
+      return NC_EBADTYPE;
+}
+
+/* Given a type in one file, find its equal (if any) in another
+ * file. It sounds so simple, but it's a real pain! */
+static int
+nc4_find_equal_type(int ncid1, nc_type xtype1, int ncid2, 
+		    nc_type *xtype2)
+{
+   NC_TYPE_INFO_T *type2, *type1;
+   NC_GRP_INFO_T *grp1, *grp2;
+   int ret;
+
+   /* Check input */
+   if (xtype1 <= NC_NAT)
+      return NC_EINVAL;
+
+   /* Handle atomic types. */
+   if (xtype1 <= NC_STRING)
+   {
+      if (xtype2)
+	 *xtype2 = xtype1;
+      return NC_NOERR;
+   }
+
+   /* Find type1 in file 1. */
+   if ((ret = nc4_find_nc4_grp(ncid1, &grp1)))
+      return ret;
+   if (!(type1 = nc4_rec_find_nc_type(grp1, xtype1)))
+      return NC_EINVAL;
+
+   /* Get group info for ncid2. */
+   if ((ret = nc4_find_nc4_grp(ncid2, &grp2)))
+      return ret;
+
+   /* Does this group or its children have the type we are searching
+    * for? */
+   if ((ret = rec_find_nc_type(ncid1, type1, ncid2, grp2, &type2)))
+      return ret;
+
+   /* Found the type, but user might not ever care. The bastard.*/
+   if (xtype2)
+      *xtype2 = type2->nc_typeid;
+   
+   return ret;
+}
+
 /* Copy an attribute from one open file to another.
 
    Special programming challenge: this function must work even if one
@@ -551,67 +782,86 @@ nc_copy_att(int ncid_in, int varid_in, const char *name,
 
    /* Find out about the attribute and allocate memory for the
       data. */
-   if ((res = nc_inq_att(ncid_in, varid_in, 
-			 name, &xtype, &len)))
+   if ((res = nc_inq_att(ncid_in, varid_in, name, &xtype, &len)))
       return res;
-   if (len)
-     if (!(data = nc_malloc(len * nc4typelen(xtype))))
-       return NC_ENOMEM;
 
+   /* Can't copy to same var in same file. */
    if (ncid_in == ncid_out && varid_in == varid_out)
       return NC_NOERR;
-
-   /* Based on the type, read and write it. */
-   switch (xtype)
+   
+   if (xtype < NC_STRING)
    {
-   case NC_BYTE:
-      res = nc_get_att_schar(ncid_in, varid_in, name, 
-			     (signed char *)data);
+      /* Handle atomic types. */
+      if (len)
+	 if (!(data = malloc(len * nc4typelen(xtype))))
+	    return NC_ENOMEM;
+
+      res = nc_get_att(ncid_in, varid_in, name, data);
       if (!res)
-	 res = nc_put_att_schar(ncid_out, varid_out, name, 
-				NC_BYTE, len, 
-				(const signed char *)data);
-      break;
-   case NC_CHAR:
-      res = nc_get_att_text(ncid_in, varid_in, name, 
-			    (char *)data);
+	 res = nc_put_att(ncid_out, varid_out, name, xtype, 
+			  len, data);
+      if (len)
+	 free(data);
+   }
+   else if (xtype == NC_STRING)
+   {
+      /* Copy string attributes. */
+      char **str_data;
+      if (!(str_data = malloc(sizeof(char *) * len)))
+	 return NC_ENOMEM;
+      res = nc_get_att_string(ncid_in, varid_in, name, str_data);
       if (!res)
-	 res = nc_put_att_text(ncid_out, varid_out, name, 
-			       len, (char *)data);
-      break;
-   case NC_SHORT:
-      res = nc_get_att_short(ncid_in, varid_in, name, 
-			     (short *)data);
-      if (!res)
-	 res = nc_put_att_short(ncid_out, varid_out, name, 
-				NC_SHORT, len, (short *)data);
-      break;
-   case NC_INT:
-      res = nc_get_att_int(ncid_in, varid_in, name, 
-			   (int *)data);
-      if (!res)
-	 res = nc_put_att_int(ncid_out, varid_out, name, 
-			      NC_INT, len, (int *)data);
-      break;
-   case NC_FLOAT:
-      res = nc_get_att_float(ncid_in, varid_in, name, 
-			     (float *)data);
-      if (!res)
-	 res = nc_put_att_float(ncid_out, varid_out, name, 
-				NC_FLOAT, len, (float *)data);
-      break;
-   case NC_DOUBLE:
-      res = nc_get_att_double(ncid_in, varid_in, name, 
-			      (double *)data);
-      if (!res)
-	 res = nc_put_att_double(ncid_out, varid_out, name, 
-				 NC_DOUBLE, len, (double *)data);
-      break;
-   default:
-      res = NC_EBADTYPE;
+	 res = nc_put_att_string(ncid_out, varid_out, name, len, 
+				 (const char **)str_data);
+      nc_free_string(len, str_data);
+      free(str_data);
+   }
+   else 
+   {
+      /* Copy user-defined type attributes. */
+      int class;
+      size_t size;
+      void *data;
+      nc_type xtype_out = NC_NAT;
+
+      /* Find out if there is an equal type in the output file. */
+      if ((res = nc4_find_equal_type(ncid_in, xtype, ncid_out, &xtype_out)))
+	 return res;
+
+      /* We found an equal type! */
+      if ((res = nc_inq_user_type(ncid_in, xtype, NULL, &size, 
+				  NULL, NULL, &class)))
+	 return res;
+      if (class == NC_VLEN) /* VLENs are different... */
+      {
+	 nc_vlen_t *vldata;
+	 int i;
+	 
+	 if (!(vldata = malloc(sizeof(nc_vlen_t) * len)))
+	    return NC_ENOMEM;
+	 if ((res = nc_get_att(ncid_in, varid_in, name, vldata)))
+	    return res;
+	 if ((res = nc_put_att(ncid_out, varid_out, name, xtype_out, 
+			       len, vldata)))
+	    return res;
+	 for (i = 0; i < len; i++)
+	    if ((res = nc_free_vlen(&vldata[i])))
+	       return res;
+	 free(vldata);
+      }
+      else
+      {
+	 if (!(data = malloc(size * len)))
+	    return NC_ENOMEM;
+	 if ((res = nc_get_att(ncid_in, varid_in, name, data)))
+	    return res;
+	 if ((res = nc_put_att(ncid_out, varid_out, name, xtype_out, 
+			       len, data)))
+	    return res;
+	 free(data);
+      }
    }
 
-   if (data) nc_free(data);
    return res;
 }
 
@@ -644,6 +894,12 @@ nc_rename_att(int ncid, int varid, const char *name,
    if ((retval = nc4_find_nc_grp_h5(ncid, &nc, &grp, &h5)))
       return retval;
 
+#ifdef USE_PNETCDF
+   /* Take care of files created/opened with parallel-netcdf library. */
+   if (nc->pnetcdf_file)
+      return ncmpi_rename_att(nc->int_ncid, varid, name, newname);
+#endif /* USE_PNETCDF */
+
    /* Handle netcdf-3 files. */
    if (!h5)
       return nc3_rename_att(nc->int_ncid, varid, name, newname);
@@ -700,11 +956,9 @@ nc_rename_att(int ncid, int varid, const char *name,
       }
       else
       {
-	 if ((retval = nc4_open_var_grp(grp, varid, &datasetid)))
+	 if ((retval = nc4_open_var_grp2(grp, varid, &datasetid)))
 	    return retval;
 	 retval = nc4_delete_hdf5_att(datasetid, att->name);
-	 if (H5Dclose(datasetid) < 0)
-	    BAIL(NC_EHDFERR);
 	 if (retval)
 	    return retval;
       }
@@ -714,8 +968,6 @@ nc_rename_att(int ncid, int varid, const char *name,
    strcpy(att->name, norm_newname);
    att->dirty = 1;
 
-  exit:
-   if (datasetid > 0) H5Dclose(datasetid);   
    return retval;
 }
 
@@ -746,6 +998,12 @@ nc_del_att(int ncid, int varid, const char *name)
    if ((retval = nc4_find_nc_grp_h5(ncid, &nc, &grp, &h5)))
       return retval;
 
+#ifdef USE_PNETCDF
+   /* Take care of files created/opened with parallel-netcdf library. */
+   if (nc->pnetcdf_file)
+      return ncmpi_del_att(nc->int_ncid, varid, name);   
+#endif /* USE_PNETCDF */
+
    /* Handle netcdf-3 files. */
    if (!h5)
       return nc3_del_att(nc->int_ncid, varid, name);   
@@ -786,9 +1044,7 @@ nc_del_att(int ncid, int varid, const char *name)
 	 return NC_ENOTVAR;
       if (var->created)
       {
-	 if ((retval = nc4_open_var_grp(grp, var->varid, &datasetid)))
-	    return retval;
-	 locid = datasetid;
+	 locid = var->hdf_datasetid;
       }
    }
 
@@ -811,7 +1067,8 @@ nc_del_att(int ncid, int varid, const char *name)
       natt->attnum--;
 
    /* Delete this attribute from this list. */
-   nc4_att_list_del(attlist, att);
+   if ((retval = nc4_att_list_del(attlist, att)))
+      BAIL(retval);
    
  exit:
    if (datasetid > 0) H5Dclose(datasetid);
@@ -841,6 +1098,43 @@ nc4_put_att_tc(int ncid, int varid, const char *name, nc_type file_type,
    if (!(nc = nc4_find_nc_file(ncid)))
       return NC_EBADID;
 
+#ifdef USE_PNETCDF
+   /* Take care of files created/opened with parallel-netcdf library. */
+   if (nc->pnetcdf_file)
+   {
+      if (mem_type == NC_UBYTE)
+	 mem_type = NC_BYTE;
+      switch(mem_type)
+      {
+	 case NC_BYTE:
+	    return ncmpi_put_att_schar(nc->int_ncid, varid, name, 
+				     file_type, len, op);
+	 case NC_CHAR:
+	    return ncmpi_put_att_text(nc->int_ncid, varid, name, 
+				    len, op);
+	 case NC_SHORT:
+	    return ncmpi_put_att_short(nc->int_ncid, varid, name, 
+				     file_type, len, op);
+	 case NC_INT:
+	    if (mem_type_is_long)
+	       return ncmpi_put_att_long(nc->int_ncid, varid, name, 
+				       file_type, len, op);
+	    else
+	       return ncmpi_put_att_int(nc->int_ncid, varid, name, 
+				      file_type, len, op);
+	 case NC_FLOAT:
+	    return ncmpi_put_att_float(nc->int_ncid, varid, name, 
+				     file_type, len, op);
+	 case NC_DOUBLE:
+	    return ncmpi_put_att_double(nc->int_ncid, varid, name, 
+				      file_type, len, op);
+	 case NC_NAT:
+	 default:
+	    return NC_EBADTYPE;
+      }
+   }
+#endif /* USE_PNETCDF */
+
    /* Handle netcdf-3 files. */
    if (!nc->nc4_info)
    {
@@ -906,6 +1200,36 @@ nc4_get_att_tc(int ncid, int varid, const char *name, nc_type mem_type,
    if (!(nc = nc4_find_nc_file(ncid)))
       return NC_EBADID;
 
+#ifdef USE_PNETCDF
+   /* Take care of files created/opened with parallel-netcdf library. */
+   if (nc->pnetcdf_file)
+   {
+      if (mem_type == NC_UBYTE)
+	 mem_type = NC_BYTE;
+      switch(mem_type)
+      {
+	 case NC_BYTE:
+	    return ncmpi_get_att_schar(nc->int_ncid, varid, name, ip);
+	 case NC_CHAR:
+	    return ncmpi_get_att_text(nc->int_ncid, varid, name, ip);
+	 case NC_SHORT:
+	    return ncmpi_get_att_short(nc->int_ncid, varid, name, ip);
+	 case NC_INT:
+	    if (mem_type_is_long)
+	       return ncmpi_get_att_long(nc->int_ncid, varid, name, ip);
+	    else
+	       return ncmpi_get_att_int(nc->int_ncid, varid, name, ip);
+	 case NC_FLOAT:
+	    return ncmpi_get_att_float(nc->int_ncid, varid, name, ip);
+	 case NC_DOUBLE:
+	    return ncmpi_get_att_double(nc->int_ncid, varid, name, ip);
+	 case NC_NAT:
+	 default:
+	    return NC_EBADTYPE;
+      }
+   }
+#endif /* USE_PNETCDF */
+
    /* Handle netcdf-3 files. */
    if (!nc->nc4_info)
    {
