@@ -1,5 +1,5 @@
 /*
- * Copyright 1993-2006 University Corporation for Atmospheric Research/Unidata
+ * Copyright 1993-2009 University Corporation for Atmospheric Research/Unidata
  * 
  * Portions of this software were developed by the Unidata Program at the 
  * University Corporation for Atmospheric Research.
@@ -30,7 +30,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-/* "$Id: netcdf_base.h,v 1.11 2008/06/10 18:55:27 ed Exp $" */
+/* "$Id: netcdf_base.h,v 1.56 2010/01/21 16:00:18 ed Exp $" */
 
 #ifndef _NETCDF_
 #define _NETCDF_
@@ -38,10 +38,12 @@
 #include <stddef.h> /* size_t, ptrdiff_t */
 #include <errno.h>  /* netcdf functions sometimes return system errors */
 /* These defs added by netCDF configure because parallel HDF5 is not present. */
+#ifndef MPI_INCLUDED
 #define MPI_Comm int
 #define MPI_Info int
 #define MPI_COMM_WORLD 0
 #define MPI_INFO_NULL 0
+#endif
 
 typedef int nc_type;
 
@@ -67,7 +69,7 @@ extern "C" {
 #define	NC_UINT64 	11	/* unsigned 8-byte int */
 #define	NC_STRING 	12	/* string */
 
-/* The folloing are use internally in support of user-defines
+/* The following are use internally in support of user-defines
  * types. They are also the class returned by nc_inq_user_type. */
 #define	NC_VLEN 	13	/* used internally for vlen types */
 #define	NC_OPAQUE 	14	/* used internally for opaque types */
@@ -122,11 +124,11 @@ extern "C" {
 #define X_UINT64_MAX    (18446744073709551615ULL)
 
 /*
- * The above values are defaults.
- * If you wish a variable to use a different value than the above
- * defaults, create an attribute with the same type as the variable
- * and the following reserved name. The value you give the attribute
- * will be used as the fill value for that variable.
+ * The above values are defaults.  If you wish a variable to use a
+ * different value than the above defaults, create an attribute with
+ * the same type as the variable and the following reserved name. The
+ * value you give the attribute will be used as the fill value for
+ * that variable.
  */
 #define _FillValue	"_FillValue"
 #define NC_FILL		0	/* argument to ncsetfill to clear NC_NOFILL */
@@ -145,7 +147,7 @@ extern "C" {
 #define NC_NOCLOBBER	0x0004	/* Don't destroy existing file on create */
 #define NC_64BIT_OFFSET 0x0200  /* Use large (64-bit) file offsets */
 #define NC_NETCDF4      0x1000  /* Use netCDF-4/HDF5 format */
-#define NC_CLASSIC_MODEL 0x0100 /* Enforce strict netcdf-3 rules. */
+#define NC_CLASSIC_MODEL 0x0100 /* Enforce classic model when used with NC_NETCDF4. */
 
 /*
  * Use these 'mode' flags for both nc_create and nc_open.
@@ -153,6 +155,7 @@ extern "C" {
 #define NC_SHARE       0x0800	/* Share updates, limit cacheing */
 #define NC_MPIIO       0x2000 
 #define NC_MPIPOSIX    0x4000
+#define NC_PNETCDF     0x8000
 
 /* The following flag currently is ignored, but use in
  * nc_open() or nc_create() may someday support use of advisory
@@ -208,6 +211,26 @@ extern "C" {
 #define NC_ENDIAN_LITTLE 1
 #define NC_ENDIAN_BIG    2
 
+/* In HDF5 files you can set storage for each variable to be either
+ * contiguous or chunked, with nc_def_var_chunking().  These defines
+ * are used there. */
+#define NC_CHUNKED    0
+#define NC_CONTIGUOUS 1
+
+/* In HDF5 files you can set check-summing for each variable.
+ * Currently the only checksum available is Fletcher-32, which can be
+ * set with the function nc_def_var_fletcher32.  These defines are used
+ * there. */
+#define NC_NOCHECKSUM 0
+#define NC_FLETCHER32 1
+
+/* In HDF5 files you can specify that a shuffle filter should be used
+ * on each chunk of a variable to improve compression for that
+ * variable.  This per-variable shuffle property can be set with the
+ * function nc_def_var_deflate.  These defines are used there. */
+#define NC_NOSHUFFLE 0
+#define NC_SHUFFLE   1
+
 /*
  * The netcdf version 3 functions all return integer error status.
  * These are the possible values, in addition to certain
@@ -258,6 +281,17 @@ extern "C" {
 
 #define NC_EAXISTYPE    (-65)   /* Unknown axis type. */
 
+/* Following errors are added for DAP */
+#define NC_EDAP         (-66)   /* Generic DAP error */
+#define NC_ECURL        (-67)   /* Generic libcurl error */
+#define NC_EIO          (-68)   /* Generic IO error */
+#define NC_ENODATA      (-69)   /* Attempt to access variable with no data */
+#define NC_EDAPSVC      (-70)   /* DAP Server side error */
+#define NC_EDAS		(-71)   /* Malformed or inaccessible DAS */
+#define NC_EDDS		(-72)   /* Malformed or inaccessible DDS */
+#define NC_EDATADDS	(-73)   /* Malformed or inaccessible DATADDS */
+#define NC_EDAPURL	(-74)   /* Malformed DAP URL */
+#define NC_EDAPCONSTRAINT (-75)   /* Malformed DAP Constraint*/
 
 /* The following was added in support of netcdf-4. Make all netcdf-4
    error codes < -100 so that errors can be added to netcdf-3 if
@@ -290,8 +324,17 @@ extern "C" {
 #define NC_ENOGRP        (-125) /* No group found. */
 #define NC_ESTORAGE      (-126) /* Can't specify both contiguous and chunking. */
 #define NC_EBADCHUNK     (-127) /* Bad chunksize. */
+#define NC_ENOTBUILT     (-128) /* Attempt to use feature that was not turned on when netCDF was built. */
 #define NC4_LAST_ERROR   (-128) 
 
+/* This is used in netCDF-4 files for dimensions without coordinate
+ * vars. */
+#define DIM_WITHOUT_VARIABLE "This is a netCDF dimension but not a netCDF variable."
+
+/* This is here at the request of the NCO team to support the stupid
+ * mistake of having chunksizes be first ints, then size_t. Doh! */
+#define NC_HAVE_NEW_CHUNKING_API 1
+
 /*
  * The Interface
  */
@@ -305,8 +348,8 @@ extern "C" {
 #   define MSC_EXTRA __declspec(dllimport)
 #  endif
 #include <io.h>
-#define lseek _lseeki64
-#define off_t __int64
+/*#define lseek _lseeki64
+  #define off_t __int64*/
 #else
 #define MSC_EXTRA
 #endif	/* defined(DLL_NETCDF) */
@@ -352,19 +395,18 @@ nc_open_par(const char *path, int mode, MPI_Comm comm, MPI_Info info,
 
 EXTERNL int
 nc_var_par_access(int ncid, int varid, int par_access);
-/*nc_var_independent(int ncid, int varid, int independent);*/
 
 /* Given an ncid and group name (NULL gets root group), return
  * locid. */
 EXTERNL int
-nc_inq_ncid(int ncid, char *name, int *grp_ncid);
+nc_inq_ncid(int ncid, const char *name, int *grp_ncid);
 
 /* Given a location id, return the number of groups it contains, and
  * an array of their locids. */
 EXTERNL int
 nc_inq_grps(int ncid, int *numgrps, int *ncids);
 
-/* Given locid, find name of group. (Root group is named "".) */
+/* Given locid, find name of group. (Root group is named "/".) */
 EXTERNL int
 nc_inq_grpname(int ncid, char *name);
 
@@ -377,24 +419,24 @@ nc_inq_grpname_full(int ncid, size_t *lenp, char *full_name);
 EXTERNL int
 nc_inq_grpname_len(int ncid, size_t *lenp);
 
-/* Given an ncid, find the ncid of it's parent group. */
+/* Given an ncid, find the ncid of its parent group. */
 EXTERNL int
 nc_inq_grp_parent(int ncid, int *parent_ncid);
 
 /* Given a name and parent ncid, find group ncid. */
 EXTERNL int
-nc_inq_grp_ncid(int ncid, char *grp_name, int *grp_ncid);
+nc_inq_grp_ncid(int ncid, const char *grp_name, int *grp_ncid);
 
 /* Given a full name and ncid, find group ncid. */
 EXTERNL int
-nc_inq_grp_full_ncid(int ncid, char *full_name, int *grp_ncid);
+nc_inq_grp_full_ncid(int ncid, const char *full_name, int *grp_ncid);
 
 /* Get a list of ids for all the variables in a group. */
 EXTERNL int 
 nc_inq_varids(int ncid, int *nvars, int *varids);
 
 /* Find all dimids for a location. This finds all dimensions in a
- * group, or any of it's parents. */
+ * group, or any of its parents. */
 EXTERNL int 
 nc_inq_dimids(int ncid, int *ndims, int *dimids, int include_parents);
 
@@ -403,31 +445,40 @@ nc_inq_dimids(int ncid, int *ndims, int *dimids, int include_parents);
 EXTERNL int 
 nc_inq_typeids(int ncid, int *ntypes, int *typeids);
 
-/* Create a group. It's ncid is returned in the new_ncid pointer. */
+/* Are two types equal? */
 EXTERNL int
-nc_def_grp(int parent_ncid, char *name, int *new_ncid);
+nc_inq_type_equal(int ncid1, nc_type typeid1, int ncid2, 
+		  nc_type typeid2, int *equal);
+
+/* Create a group. its ncid is returned in the new_ncid pointer. */
+EXTERNL int
+nc_def_grp(int parent_ncid, const char *name, int *new_ncid);
 
 /* Here are functions for dealing with compound types. */
 
 /* Create a compound type. */
 EXTERNL int
-nc_def_compound(int ncid, size_t size, char *name, nc_type *typeidp);
+nc_def_compound(int ncid, size_t size, const char *name, nc_type *typeidp);
 
 /* Insert a named field into a compound type. */
 EXTERNL int
-nc_insert_compound(int ncid, nc_type xtype, char *name, 
+nc_insert_compound(int ncid, nc_type xtype, const char *name, 
 		   size_t offset, nc_type field_typeid);
 
 /* Insert a named array into a compound type. */
 EXTERNL int
-nc_insert_array_compound(int ncid, nc_type xtype, char *name, 
+nc_insert_array_compound(int ncid, nc_type xtype, const char *name, 
 			 size_t offset, nc_type field_typeid,
-			 int ndims, int *dim_sizes);
+			 int ndims, const int *dim_sizes);
 
 /* Get the name and size of a type. */
 EXTERNL int
 nc_inq_type(int ncid, nc_type xtype, char *name, size_t *size);
 
+/* Get the id of a type from the name. */
+EXTERNL int
+nc_inq_typeid(int ncid, const char *name, nc_type *typeidp);
+
 /* Get the name, size, and number of fields in a compound type. */
 EXTERNL int
 nc_inq_compound(int ncid, nc_type xtype, char *name, size_t *sizep, 
@@ -458,7 +509,7 @@ nc_inq_compound_fieldname(int ncid, nc_type xtype, int fieldid,
 
 /* Given the xtype and the name, get the fieldid. */
 EXTERNL int
-nc_inq_compound_fieldindex(int ncid, nc_type xtype, char *name, 
+nc_inq_compound_fieldindex(int ncid, nc_type xtype, const char *name, 
 			   int *fieldidp);
 
 /* Given the xtype and fieldid, get the offset. */
@@ -497,7 +548,7 @@ typedef struct {
 
 /* Create a variable length type. */
 EXTERNL int
-nc_def_vlen(int ncid, char *name, nc_type base_typeid, nc_type *xtypep);
+nc_def_vlen(int ncid, const char *name, nc_type base_typeid, nc_type *xtypep);
 
 /* Find out about a vlen. */
 EXTERNL int
@@ -511,13 +562,16 @@ nc_inq_vlen(int ncid, nc_type xtype, char *name, size_t *datum_sizep,
 EXTERNL int
 nc_free_vlen(nc_vlen_t *vl);
 
+EXTERNL int
+nc_free_vlens(size_t len, nc_vlen_t vlens[]);
+
 /* Put or get one element in a vlen array. */
 EXTERNL int
 nc_put_vlen_element(int ncid, int typeid1, void *vlen_element, 
-		    size_t len, void *data);
+		    size_t len, const void *data);
 
 EXTERNL int
-nc_get_vlen_element(int ncid, int typeid1, void *vlen_element, 
+nc_get_vlen_element(int ncid, int typeid1, const void *vlen_element, 
 		    size_t *len, void *data);
    
 /* When you read the string type the library will allocate the storage
@@ -555,7 +609,7 @@ EXTERNL int
 nc_insert_enum(int ncid, nc_type xtype, const char *name, 
 	       const void *value);
 
-/* Get information about an enum type: it's name, base type and the
+/* Get information about an enum type: its name, base type and the
  * number of members defined. */
 EXTERNL int
 nc_inq_enum(int ncid, nc_type xtype, char *name, nc_type *base_nc_typep, 
@@ -576,7 +630,7 @@ nc_inq_enum_ident(int ncid, nc_type xtype, long long value, char *identifier);
 
 /* Create an opaque type. Provide a size and a name. */
 EXTERNL int
-nc_def_opaque(int ncid, size_t size, char *name, nc_type *xtypep);
+nc_def_opaque(int ncid, size_t size, const char *name, nc_type *xtypep);
 
 /* Get information about an opaque type. */
 EXTERNL int
@@ -646,28 +700,32 @@ EXTERNL int
 nc_inq_var_deflate(int ncid, int varid, int *shufflep, 
 		   int *deflatep, int *deflate_levelp);
 
-/* Set fletcher32 checksum for a var. This must be done after
-   nc_def_var and before nc_enddef. */
+/* Find out szip settings of a var. */
+EXTERNL int
+nc_inq_var_szip(int ncid, int varid, int *options_maskp, int *pixels_per_blockp);
+
+/* Set fletcher32 checksum for a var. This must be done after nc_def_var
+   and before nc_enddef. */
 EXTERNL int
 nc_def_var_fletcher32(int ncid, int varid, int fletcher32);
    
-/* Inq fletcher32 checksum for a var. */
+/* Inquire about fletcher32 checksum for a var. */
 EXTERNL int
 nc_inq_var_fletcher32(int ncid, int varid, int *fletcher32p);
 
 /* Define chunking for a variable. This must be done after nc_def_var
    and before nc_enddef. */
 EXTERNL int
-nc_def_var_chunking(int ncid, int varid, int contiguous, int *chunksizesp);
+nc_def_var_chunking(int ncid, int varid, int storage, const size_t *chunksizesp);
 
 /* Inq chunking stuff for a var. */
 EXTERNL int
-nc_inq_var_chunking(int ncid, int varid, int *contiguousp, int *chunksizesp);
+nc_inq_var_chunking(int ncid, int varid, int *storagep, size_t *chunksizesp);
 
 /* Define fill value behavior for a variable. This must be done after
    nc_def_var and before nc_enddef. */
 EXTERNL int
-nc_def_var_fill(int ncid, int varid, int no_fill, void *fill_value);
+nc_def_var_fill(int ncid, int varid, int no_fill, const void *fill_value);
 
 /* Inq fill value setting for a var. */
 EXTERNL int
@@ -675,11 +733,11 @@ nc_inq_var_fill(int ncid, int varid, int *no_fill, void *fill_value);
 
 /* Define the endianness of a variable. */
 EXTERNL int
-nc_def_var_endian(int ncid, int varid, int endinan);
+nc_def_var_endian(int ncid, int varid, int endian);
 
 /* Learn about the endianness of a variable. */
 EXTERNL int
-nc_inq_var_endian(int ncid, int varid, int *endinanp);
+nc_inq_var_endian(int ncid, int varid, int *endianp);
 
 /* Set the fill mode (classic or 64-bit offset files only). */
 EXTERNL int
@@ -690,6 +748,24 @@ nc_set_fill(int ncid, int fillmode, int *old_modep);
 EXTERNL int
 nc_set_default_format(int format, int *old_formatp);
 
+/* Set the cache size, nelems, and preemption policy. */
+EXTERNL int
+nc_set_chunk_cache(size_t size, size_t nelems, float preemption);
+
+/* Get the cache size, nelems, and preemption policy. */
+EXTERNL int
+nc_get_chunk_cache(size_t *sizep, size_t *nelemsp, float *preemptionp);
+
+/* Set the per-variable cache size, nelems, and preemption policy. */
+EXTERNL int
+nc_set_var_chunk_cache(int ncid, int varid, size_t size, size_t nelems, 
+		       float preemption);
+
+/* Set the per-variable cache size, nelems, and preemption policy. */
+EXTERNL int
+nc_get_var_chunk_cache(int ncid, int varid, size_t *sizep, size_t *nelemsp, 
+		       float *preemptionp);
+
 EXTERNL int
 nc_redef(int ncid);
 
@@ -1517,13 +1593,6 @@ EXTERNL int
 nc_get_var_string(int ncid, int varid, char **ip);
 
 #ifdef LOGGING
-#ifdef EXTRA_MEM_DEBUG
-
-/* When using EXTRA_MEM_DEBUG this will total up allocs and frees
- * and make sure everything adds up. Not to be used by users. */
-EXTERNL void
-nc_exit(void);
-#endif /* EXTRA_MEM_DEBUG */
 
 /* Set the log level. 0 shows only errors, 1 only major messages,
  * etc., to 5, which shows way too much information. */
@@ -1617,14 +1686,6 @@ EXTERNL int ncerr;
 #define	NC_EXDR		(-32)	/* */
 #define	NC_SYSERR	(-31)
 
-/*
- * Avoid use of this meaningless macro
- * Use sysconf(_SC_OPEN_MAX).
- */
-#ifndef MAX_NC_OPEN
-#define MAX_NC_OPEN 32
-#endif
-
 /*
  * Global options variable.
  * Used to determine behavior of error handler.
