@@ -1,5 +1,5 @@
 /*
- * Copyright 1993-2006 University Corporation for Atmospheric Research/Unidata
+ * Copyright 1993-2009 University Corporation for Atmospheric Research/Unidata
  * 
  * Portions of this software were developed by the Unidata Program at the 
  * University Corporation for Atmospheric Research.
@@ -30,7 +30,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-/* "$Id: netcdf_base.h,v 1.28 2009/02/26 02:55:05 dmh Exp $" */
+/* "$Id: netcdf_base.h,v 1.56 2010/01/21 16:00:18 ed Exp $" */
 
 #ifndef _NETCDF_
 #define _NETCDF_
@@ -62,7 +62,7 @@ extern "C" {
 #define	NC_UINT64 	11	/* unsigned 8-byte int */
 #define	NC_STRING 	12	/* string */
 
-/* The folloing are use internally in support of user-defines
+/* The following are use internally in support of user-defines
  * types. They are also the class returned by nc_inq_user_type. */
 #define	NC_VLEN 	13	/* used internally for vlen types */
 #define	NC_OPAQUE 	14	/* used internally for opaque types */
@@ -116,10 +116,6 @@ extern "C" {
 #define X_INT64_MIN     (-X_INT64_MAX - 1)
 #define X_UINT64_MAX    (18446744073709551615ULL)
 
-/* These are limits for default chunksizes. (2^16 and 2^20). */
-#define NC_LEN_TOO_BIG 65536 
-#define NC_LEN_WAY_TOO_BIG 1048576
-
 /*
  * The above values are defaults.  If you wish a variable to use a
  * different value than the above defaults, create an attribute with
@@ -152,6 +148,7 @@ extern "C" {
 #define NC_SHARE       0x0800	/* Share updates, limit cacheing */
 #define NC_MPIIO       0x2000 
 #define NC_MPIPOSIX    0x4000
+#define NC_PNETCDF     0x8000
 
 /* The following flag currently is ignored, but use in
  * nc_open() or nc_create() may someday support use of advisory
@@ -281,6 +278,13 @@ extern "C" {
 #define NC_EDAP         (-66)   /* Generic DAP error */
 #define NC_ECURL        (-67)   /* Generic libcurl error */
 #define NC_EIO          (-68)   /* Generic IO error */
+#define NC_ENODATA      (-69)   /* Attempt to access variable with no data */
+#define NC_EDAPSVC      (-70)   /* DAP Server side error */
+#define NC_EDAS		(-71)   /* Malformed or inaccessible DAS */
+#define NC_EDDS		(-72)   /* Malformed or inaccessible DDS */
+#define NC_EDATADDS	(-73)   /* Malformed or inaccessible DATADDS */
+#define NC_EDAPURL	(-74)   /* Malformed DAP URL */
+#define NC_EDAPCONSTRAINT (-75)   /* Malformed DAP Constraint*/
 
 /* The following was added in support of netcdf-4. Make all netcdf-4
    error codes < -100 so that errors can be added to netcdf-3 if
@@ -313,8 +317,17 @@ extern "C" {
 #define NC_ENOGRP        (-125) /* No group found. */
 #define NC_ESTORAGE      (-126) /* Can't specify both contiguous and chunking. */
 #define NC_EBADCHUNK     (-127) /* Bad chunksize. */
+#define NC_ENOTBUILT     (-128) /* Attempt to use feature that was not turned on when netCDF was built. */
 #define NC4_LAST_ERROR   (-128) 
 
+/* This is used in netCDF-4 files for dimensions without coordinate
+ * vars. */
+#define DIM_WITHOUT_VARIABLE "This is a netCDF dimension but not a netCDF variable."
+
+/* This is here at the request of the NCO team to support the stupid
+ * mistake of having chunksizes be first ints, then size_t. Doh! */
+#define NC_HAVE_NEW_CHUNKING_API 1
+
 /*
  * The Interface
  */
@@ -379,14 +392,14 @@ nc_var_par_access(int ncid, int varid, int par_access);
 /* Given an ncid and group name (NULL gets root group), return
  * locid. */
 EXTERNL int
-nc_inq_ncid(int ncid, char *name, int *grp_ncid);
+nc_inq_ncid(int ncid, const char *name, int *grp_ncid);
 
 /* Given a location id, return the number of groups it contains, and
  * an array of their locids. */
 EXTERNL int
 nc_inq_grps(int ncid, int *numgrps, int *ncids);
 
-/* Given locid, find name of group. (Root group is named "".) */
+/* Given locid, find name of group. (Root group is named "/".) */
 EXTERNL int
 nc_inq_grpname(int ncid, char *name);
 
@@ -405,11 +418,11 @@ nc_inq_grp_parent(int ncid, int *parent_ncid);
 
 /* Given a name and parent ncid, find group ncid. */
 EXTERNL int
-nc_inq_grp_ncid(int ncid, char *grp_name, int *grp_ncid);
+nc_inq_grp_ncid(int ncid, const char *grp_name, int *grp_ncid);
 
 /* Given a full name and ncid, find group ncid. */
 EXTERNL int
-nc_inq_grp_full_ncid(int ncid, char *full_name, int *grp_ncid);
+nc_inq_grp_full_ncid(int ncid, const char *full_name, int *grp_ncid);
 
 /* Get a list of ids for all the variables in a group. */
 EXTERNL int 
@@ -425,31 +438,40 @@ nc_inq_dimids(int ncid, int *ndims, int *dimids, int include_parents);
 EXTERNL int 
 nc_inq_typeids(int ncid, int *ntypes, int *typeids);
 
+/* Are two types equal? */
+EXTERNL int
+nc_inq_type_equal(int ncid1, nc_type typeid1, int ncid2, 
+		  nc_type typeid2, int *equal);
+
 /* Create a group. its ncid is returned in the new_ncid pointer. */
 EXTERNL int
-nc_def_grp(int parent_ncid, char *name, int *new_ncid);
+nc_def_grp(int parent_ncid, const char *name, int *new_ncid);
 
 /* Here are functions for dealing with compound types. */
 
 /* Create a compound type. */
 EXTERNL int
-nc_def_compound(int ncid, size_t size, char *name, nc_type *typeidp);
+nc_def_compound(int ncid, size_t size, const char *name, nc_type *typeidp);
 
 /* Insert a named field into a compound type. */
 EXTERNL int
-nc_insert_compound(int ncid, nc_type xtype, char *name, 
+nc_insert_compound(int ncid, nc_type xtype, const char *name, 
 		   size_t offset, nc_type field_typeid);
 
 /* Insert a named array into a compound type. */
 EXTERNL int
-nc_insert_array_compound(int ncid, nc_type xtype, char *name, 
+nc_insert_array_compound(int ncid, nc_type xtype, const char *name, 
 			 size_t offset, nc_type field_typeid,
-			 int ndims, int *dim_sizes);
+			 int ndims, const int *dim_sizes);
 
 /* Get the name and size of a type. */
 EXTERNL int
 nc_inq_type(int ncid, nc_type xtype, char *name, size_t *size);
 
+/* Get the id of a type from the name. */
+EXTERNL int
+nc_inq_typeid(int ncid, const char *name, nc_type *typeidp);
+
 /* Get the name, size, and number of fields in a compound type. */
 EXTERNL int
 nc_inq_compound(int ncid, nc_type xtype, char *name, size_t *sizep, 
@@ -480,7 +502,7 @@ nc_inq_compound_fieldname(int ncid, nc_type xtype, int fieldid,
 
 /* Given the xtype and the name, get the fieldid. */
 EXTERNL int
-nc_inq_compound_fieldindex(int ncid, nc_type xtype, char *name, 
+nc_inq_compound_fieldindex(int ncid, nc_type xtype, const char *name, 
 			   int *fieldidp);
 
 /* Given the xtype and fieldid, get the offset. */
@@ -519,7 +541,7 @@ typedef struct {
 
 /* Create a variable length type. */
 EXTERNL int
-nc_def_vlen(int ncid, char *name, nc_type base_typeid, nc_type *xtypep);
+nc_def_vlen(int ncid, const char *name, nc_type base_typeid, nc_type *xtypep);
 
 /* Find out about a vlen. */
 EXTERNL int
@@ -533,13 +555,16 @@ nc_inq_vlen(int ncid, nc_type xtype, char *name, size_t *datum_sizep,
 EXTERNL int
 nc_free_vlen(nc_vlen_t *vl);
 
+EXTERNL int
+nc_free_vlens(size_t len, nc_vlen_t vlens[]);
+
 /* Put or get one element in a vlen array. */
 EXTERNL int
 nc_put_vlen_element(int ncid, int typeid1, void *vlen_element, 
-		    size_t len, void *data);
+		    size_t len, const void *data);
 
 EXTERNL int
-nc_get_vlen_element(int ncid, int typeid1, void *vlen_element, 
+nc_get_vlen_element(int ncid, int typeid1, const void *vlen_element, 
 		    size_t *len, void *data);
    
 /* When you read the string type the library will allocate the storage
@@ -598,7 +623,7 @@ nc_inq_enum_ident(int ncid, nc_type xtype, long long value, char *identifier);
 
 /* Create an opaque type. Provide a size and a name. */
 EXTERNL int
-nc_def_opaque(int ncid, size_t size, char *name, nc_type *xtypep);
+nc_def_opaque(int ncid, size_t size, const char *name, nc_type *xtypep);
 
 /* Get information about an opaque type. */
 EXTERNL int
@@ -668,6 +693,10 @@ EXTERNL int
 nc_inq_var_deflate(int ncid, int varid, int *shufflep, 
 		   int *deflatep, int *deflate_levelp);
 
+/* Find out szip settings of a var. */
+EXTERNL int
+nc_inq_var_szip(int ncid, int varid, int *options_maskp, int *pixels_per_blockp);
+
 /* Set fletcher32 checksum for a var. This must be done after nc_def_var
    and before nc_enddef. */
 EXTERNL int
@@ -680,7 +709,7 @@ nc_inq_var_fletcher32(int ncid, int varid, int *fletcher32p);
 /* Define chunking for a variable. This must be done after nc_def_var
    and before nc_enddef. */
 EXTERNL int
-nc_def_var_chunking(int ncid, int varid, int storage, size_t *chunksizesp);
+nc_def_var_chunking(int ncid, int varid, int storage, const size_t *chunksizesp);
 
 /* Inq chunking stuff for a var. */
 EXTERNL int
@@ -689,7 +718,7 @@ nc_inq_var_chunking(int ncid, int varid, int *storagep, size_t *chunksizesp);
 /* Define fill value behavior for a variable. This must be done after
    nc_def_var and before nc_enddef. */
 EXTERNL int
-nc_def_var_fill(int ncid, int varid, int no_fill, void *fill_value);
+nc_def_var_fill(int ncid, int varid, int no_fill, const void *fill_value);
 
 /* Inq fill value setting for a var. */
 EXTERNL int
@@ -720,6 +749,16 @@ nc_set_chunk_cache(size_t size, size_t nelems, float preemption);
 EXTERNL int
 nc_get_chunk_cache(size_t *sizep, size_t *nelemsp, float *preemptionp);
 
+/* Set the per-variable cache size, nelems, and preemption policy. */
+EXTERNL int
+nc_set_var_chunk_cache(int ncid, int varid, size_t size, size_t nelems, 
+		       float preemption);
+
+/* Set the per-variable cache size, nelems, and preemption policy. */
+EXTERNL int
+nc_get_var_chunk_cache(int ncid, int varid, size_t *sizep, size_t *nelemsp, 
+		       float *preemptionp);
+
 EXTERNL int
 nc_redef(int ncid);
 
