@@ -3,7 +3,7 @@
    See COPYRIGHT file for conditions of use.
 
    Test netcdf-4 variables. 
-   $Id: tst_vars.c,v 1.33 2008/05/15 19:42:43 ed Exp $
+   $Id: tst_vars.c,v 1.49 2009/12/30 12:03:48 ed Exp $
 */
 
 #include <nc_tests.h>
@@ -92,9 +92,6 @@ create_4D_example(char *file_name, int cmode)
    /* Loop indexes. */
    int lvl, lat, lon, rec, i = 0;
    
-   /* Error handling. */
-   int retval;
-
    /* Create some pretend data. If this wasn't an example program, we
     * would have some real data to write, for example, model
     * output. */
@@ -112,20 +109,15 @@ create_4D_example(char *file_name, int cmode)
 	 }
 
    /* Create the file. */
-   if ((retval = nc_create(file_name, cmode, &ncid)))
-      ERR;
+   if (nc_create(file_name, cmode, &ncid)) ERR;
 
    /* Define the dimensions. The record dimension is defined to have
     * unlimited length - it can grow as needed. In this example it is
     * the time dimension.*/
-   if ((retval = nc_def_dim(ncid, LVL_NAME, NLVL, &lvl_dimid)))
-      ERR;
-   if ((retval = nc_def_dim(ncid, LAT_NAME, NLAT, &lat_dimid)))
-      ERR;
-   if ((retval = nc_def_dim(ncid, LON_NAME, NLON, &lon_dimid)))
-      ERR;
-   if ((retval = nc_def_dim(ncid, REC_NAME, NC_UNLIMITED, &rec_dimid)))
-      ERR;
+   if (nc_def_dim(ncid, LVL_NAME, NLVL, &lvl_dimid)) ERR;
+   if (nc_def_dim(ncid, LAT_NAME, NLAT, &lat_dimid)) ERR;
+   if (nc_def_dim(ncid, LON_NAME, NLON, &lon_dimid)) ERR;
+   if (nc_def_dim(ncid, REC_NAME, NC_UNLIMITED, &rec_dimid)) ERR;
 
    /* Define the coordinate variables. We will only define coordinate
       variables for lat and lon.  Ordinarily we would need to provide
@@ -133,20 +125,16 @@ create_4D_example(char *file_name, int cmode)
       since coordinate variables only have one dimension, we can
       simply provide the address of that dimension ID (&lat_dimid) and
       similarly for (&lon_dimid). */
-   if ((retval = nc_def_var(ncid, LAT_NAME, NC_FLOAT, 1, &lat_dimid, 
-			    &lat_varid)))
-      ERR;
-   if ((retval = nc_def_var(ncid, LON_NAME, NC_FLOAT, 1, &lon_dimid, 
-			    &lon_varid)))
-      ERR;
+   if (nc_def_var(ncid, LAT_NAME, NC_FLOAT, 1, &lat_dimid, 
+			    &lat_varid)) ERR;
+   if (nc_def_var(ncid, LON_NAME, NC_FLOAT, 1, &lon_dimid, 
+			    &lon_varid)) ERR;
 
    /* Assign units attributes to coordinate variables. */
-   if ((retval = nc_put_att_text(ncid, lat_varid, UNITS, 
-				 strlen(DEGREES_NORTH), DEGREES_NORTH)))
-      ERR;
-   if ((retval = nc_put_att_text(ncid, lon_varid, UNITS, 
-				 strlen(DEGREES_EAST), DEGREES_EAST)))
-      ERR;
+   if (nc_put_att_text(ncid, lat_varid, UNITS, 
+				 strlen(DEGREES_NORTH), DEGREES_NORTH)) ERR;
+   if (nc_put_att_text(ncid, lon_varid, UNITS, 
+				 strlen(DEGREES_EAST), DEGREES_EAST)) ERR;
 
    /* The dimids array is used to pass the dimids of the dimensions of
       the netCDF variables. Both of the netCDF variables we are
@@ -159,31 +147,24 @@ create_4D_example(char *file_name, int cmode)
 
    /* Define the netCDF variables for the pressure and temperature
     * data. */
-   if ((retval = nc_def_var(ncid, PRES_NAME, NC_FLOAT, NDIMS_EX, 
-			    dimids, &pres_varid)))
-      ERR;
-   if ((retval = nc_def_var(ncid, TEMP_NAME, NC_FLOAT, NDIMS_EX, 
-			    dimids, &temp_varid)))
-      ERR;
+   if (nc_def_var(ncid, PRES_NAME, NC_FLOAT, NDIMS_EX, 
+			    dimids, &pres_varid)) ERR;
+   if (nc_def_var(ncid, TEMP_NAME, NC_FLOAT, NDIMS_EX, 
+			    dimids, &temp_varid)) ERR;
 
    /* Assign units attributes to the netCDF variables. */
-   if ((retval = nc_put_att_text(ncid, pres_varid, UNITS, 
-				 strlen(PRES_UNITS), PRES_UNITS)))
-      ERR;
-   if ((retval = nc_put_att_text(ncid, temp_varid, UNITS, 
-				 strlen(TEMP_UNITS), TEMP_UNITS)))
-      ERR;
+   if (nc_put_att_text(ncid, pres_varid, UNITS, 
+				 strlen(PRES_UNITS), PRES_UNITS)) ERR;
+   if (nc_put_att_text(ncid, temp_varid, UNITS, 
+				 strlen(TEMP_UNITS), TEMP_UNITS)) ERR;
 
    /* End define mode. */
-   if ((retval = nc_enddef(ncid)))
-      ERR;
+   if (nc_enddef(ncid)) ERR;
 
    /* Write the coordinate variable data. This will put the latitudes
       and longitudes of our data grid into the netCDF file. */
-   if ((retval = nc_put_var_float(ncid, lat_varid, &lats[0])))
-      ERR;
-   if ((retval = nc_put_var_float(ncid, lon_varid, &lons[0])))
-      ERR;
+   if (nc_put_var_float(ncid, lat_varid, &lats[0])) ERR;
+   if (nc_put_var_float(ncid, lon_varid, &lons[0])) ERR;
 
    /* These settings tell netcdf to write one timestep of data. (The
       setting of start[0] inside the loop below tells netCDF which
@@ -203,17 +184,14 @@ create_4D_example(char *file_name, int cmode)
    for (rec = 0; rec < NREC; rec++)
    {
       start[0] = rec;
-      if ((retval = nc_put_vara_float(ncid, pres_varid, start, count, 
-				      &pres_out[0][0][0])))
-	 ERR;
-      if ((retval = nc_put_vara_float(ncid, temp_varid, start, count, 
-				      &temp_out[0][0][0])))
-	 ERR;
+      if (nc_put_vara_float(ncid, pres_varid, start, count, 
+				      &pres_out[0][0][0])) ERR;
+      if (nc_put_vara_float(ncid, temp_varid, start, count, 
+				      &temp_out[0][0][0])) ERR;
    }
 
    /* Close the file. */
-   if ((retval = nc_close(ncid)))
-      ERR;
+   if (nc_close(ncid)) ERR;
 
    return err;
 }
@@ -312,10 +290,6 @@ check_4D_example(char *file_name, int expected_format)
 int
 main(int argc, char **argv)
 {
-#ifdef USE_PARALLEL
-   MPI_Init(&argc, &argv);
-#endif
-
    int ncid, dimids[3];
    int char_varid, byte_varid, ubyte_varid, short_varid, int_varid, float_varid, double_varid;
    int ushort_varid, uint_varid, int64_varid, uint64_varid;
@@ -334,8 +308,6 @@ main(int argc, char **argv)
    unsigned long long uint64_out[DIM1_LEN][DIM2_LEN] = {{0, 10101, 9999999},{0, 10101, 9999999}};
    char char_out[DIM1_LEN][DIM2_LEN][DIM3_LEN] = {{"lalala", "lololo", "lelele"}, {"lalala", "lololo", "lelele"}};
 
-   /*nc_set_log_level(4);*/
-
    printf("\n*** Testing netcdf-4 variable functions.\n");
 
    printf("*** testing netcdf-4 varids inq on netcdf-3 file...");
@@ -397,6 +369,7 @@ main(int argc, char **argv)
 	 nc_type xtype_in;
 	 int ndims_in, dimids_in[10], natts_in, varid_in;
 	 char name_in[NC_MAX_NAME+1];
+	 size_t size_in;
 
 	 if (nc_open(FILE_NAME, 0, &ncid)) ERR;
 	 if (nc_inq_var(ncid, 0, name_in, &xtype_in, &ndims_in, dimids_in, 
@@ -422,6 +395,58 @@ main(int argc, char **argv)
 	 if (xtype_in != NC_CHAR) ERR;
 	 if (nc_inq_vartype(ncid, 2, &xtype_in)) ERR;      
 	 if (xtype_in != NC_SHORT) ERR;
+
+	 /* Check inquire of atomic types */
+	 if (nc_inq_type(ncid, NC_BYTE, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "byte") || size_in != sizeof(char)) ERR;
+	 if (nc_inq_type(ncid, NC_CHAR, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "char") || size_in != sizeof(char)) ERR;
+	 if (nc_inq_type(ncid, NC_SHORT, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "short") || size_in != sizeof(short)) ERR;
+	 if (nc_inq_type(ncid, NC_INT, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "int") || size_in != sizeof(int)) ERR;
+	 if (nc_inq_type(ncid, NC_FLOAT, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "float") || size_in != sizeof(float)) ERR;
+	 if (nc_inq_type(ncid, NC_DOUBLE, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "double") || size_in != sizeof(double)) ERR;
+	 if (nc_inq_type(ncid, NC_UBYTE, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "ubyte") || size_in != sizeof(unsigned char)) ERR;
+	 if (nc_inq_type(ncid, NC_USHORT, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "ushort") || size_in != sizeof(unsigned short)) ERR;
+	 if (nc_inq_type(ncid, NC_UINT, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "uint") || size_in != sizeof(unsigned int)) ERR;
+	 if (nc_inq_type(ncid, NC_INT64, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "int64") || size_in != sizeof(long long)) ERR;
+	 if (nc_inq_type(ncid, NC_UINT64, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "uint64") || size_in != sizeof(unsigned long long)) ERR;
+	 if (nc_inq_type(ncid, NC_STRING, name_in, &size_in)) ERR;
+	 if (strcmp(name_in, "string") || size_in != sizeof(char *)) ERR;
+
+	 if (nc_inq_typeid(ncid, "byte", &xtype_in)) ERR;
+	 if (xtype_in != NC_BYTE) ERR;
+	 if (nc_inq_typeid(ncid, "char", &xtype_in)) ERR;
+	 if (xtype_in != NC_CHAR) ERR;
+	 if (nc_inq_typeid(ncid, "short", &xtype_in)) ERR;
+	 if (xtype_in != NC_SHORT) ERR;
+	 if (nc_inq_typeid(ncid, "int", &xtype_in)) ERR;
+	 if (xtype_in != NC_INT) ERR;
+	 if (nc_inq_typeid(ncid, "float", &xtype_in)) ERR;
+	 if (xtype_in != NC_FLOAT) ERR;
+	 if (nc_inq_typeid(ncid, "double", &xtype_in)) ERR;
+	 if (xtype_in != NC_DOUBLE) ERR;
+	 if (nc_inq_typeid(ncid, "ubyte", &xtype_in)) ERR;
+	 if (xtype_in != NC_UBYTE) ERR;
+	 if (nc_inq_typeid(ncid, "ushort", &xtype_in)) ERR;
+	 if (xtype_in != NC_USHORT) ERR;
+	 if (nc_inq_typeid(ncid, "uint", &xtype_in)) ERR;
+	 if (xtype_in != NC_UINT) ERR;
+	 if (nc_inq_typeid(ncid, "int64", &xtype_in)) ERR;
+	 if (xtype_in != NC_INT64) ERR;
+	 if (nc_inq_typeid(ncid, "uint64", &xtype_in)) ERR;
+	 if (xtype_in != NC_UINT64) ERR;
+	 if (nc_inq_typeid(ncid, "string", &xtype_in)) ERR;
+	 if (xtype_in != NC_STRING) ERR;
+
 	 if (nc_close(ncid)) ERR;
       }
 
@@ -561,27 +586,27 @@ main(int argc, char **argv)
       if (nc_def_dim(ncid, DIM2_NAME, DIM2_LEN, &dimids[1])) ERR;
       if (nc_def_dim(ncid, DIM3_NAME, DIM3_LEN, &dimids[2])) ERR;
       if (nc_def_var(ncid, VAR_BYTE_NAME, NC_BYTE, 2, dimids, &byte_varid)) ERR;
-      if (nc_def_var_deflate(ncid, byte_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, byte_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_CHAR_NAME, NC_CHAR, 3, dimids, &char_varid)) ERR;
-      if (nc_def_var_deflate(ncid, byte_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, byte_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_SHORT_NAME, NC_SHORT, 2, dimids, &short_varid)) ERR;
-      if (nc_def_var_deflate(ncid, short_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, short_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_INT_NAME, NC_INT, 2, dimids, &int_varid)) ERR;
-      if (nc_def_var_deflate(ncid, int_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, int_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_FLOAT_NAME, NC_FLOAT, 2, dimids, &float_varid)) ERR;
-      if (nc_def_var_deflate(ncid, float_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, float_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_DOUBLE_NAME, NC_DOUBLE, 2, dimids, &double_varid)) ERR;
-      if (nc_def_var_deflate(ncid, double_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, double_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_UBYTE_NAME, NC_UBYTE, 2, dimids, &ubyte_varid)) ERR;
-      if (nc_def_var_deflate(ncid, ubyte_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, ubyte_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_USHORT_NAME, NC_USHORT, 2, dimids, &ushort_varid)) ERR;
-      if (nc_def_var_deflate(ncid, ushort_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, ushort_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_UINT_NAME, NC_UINT, 2, dimids, &uint_varid)) ERR;
-      if (nc_def_var_deflate(ncid, uint_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, uint_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_INT64_NAME, NC_INT64, 2, dimids, &int64_varid)) ERR;
-      if (nc_def_var_deflate(ncid, int64_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, int64_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
       if (nc_def_var(ncid, VAR_UINT64_NAME, NC_UINT64, 2, dimids, &uint64_varid)) ERR;
-      if (nc_def_var_deflate(ncid, uint64_varid, 0, 1, DEFLATE_LEVEL_4)) ERR;
+      if (nc_def_var_deflate(ncid, uint64_varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL_4)) ERR;
 
       if (nc_put_var_schar(ncid, byte_varid, (signed char *)byte_out)) ERR;
       if (nc_put_var_text(ncid, char_varid, (char *)char_out)) ERR;
@@ -777,7 +802,7 @@ main(int argc, char **argv)
       if (nc_def_dim(ncid, DIM4_NAME, DIM4_LEN, &dimids[0])) ERR;
       if (dimids[0] != 0) ERR;
       if (nc_def_var(ncid, VAR_NAME4, NC_INT64, NDIMS4, dimids, &varid)) ERR;
-      if (nc_def_var_deflate(ncid, varid, 0, 1, DEFLATE_LEVEL)) ERR;
+      if (nc_def_var_deflate(ncid, varid, NC_NOSHUFFLE, 1, DEFLATE_LEVEL)) ERR;
       if (varid != 0) ERR;
 
       /* Check stuff. */
@@ -793,7 +818,7 @@ main(int argc, char **argv)
 	  ndims != 1 || natts != 0 || dimids_in[0] != 0) ERR;
       if (nc_inq_var_deflate(ncid, 0, &shuffle_in, &deflate_in, 
 			     &deflate_level)) ERR;
-      if (shuffle_in ||!deflate_in || 
+      if (shuffle_in != NC_NOSHUFFLE ||!deflate_in || 
 	  deflate_level != DEFLATE_LEVEL) ERR;
 
       if (nc_close(ncid)) ERR;
@@ -813,7 +838,7 @@ main(int argc, char **argv)
 	  ndims != 1 || natts != 0 || dimids_in[0] != 0) ERR;
       if (nc_inq_var_deflate(ncid, 0, &shuffle_in, &deflate_in, 
 			     &deflate_level)) ERR;
-      if (shuffle_in ||!deflate_in || 
+      if (shuffle_in != NC_NOSHUFFLE ||!deflate_in || 
 	  deflate_level != DEFLATE_LEVEL) ERR;
 
       if (nc_close(ncid)) ERR;
@@ -853,7 +878,7 @@ main(int argc, char **argv)
 	 if (nc_def_var(ncid, var_name[i], NC_INT64, NDIMS5, dimids, 
 			&varid[i])) ERR;
 	 if (varid[i] != i) ERR;
-	 if (nc_def_var_deflate(ncid, varid[i], 1, 1, deflate_level[i])) ERR;
+	 if (nc_def_var_deflate(ncid, varid[i], NC_SHUFFLE, 1, deflate_level[i])) ERR;
       }
 
       /* Check stuff. */
@@ -871,7 +896,7 @@ main(int argc, char **argv)
 	     ndims != 1 || natts != 0 || dimids_in[0] != 0) ERR;
 	 if (nc_inq_var_deflate(ncid, varid[i], &shuffle_in, &deflate_in, 
 				&deflate_level_in)) ERR;
-	 if (!shuffle_in || !deflate_in || 
+	 if (shuffle_in != NC_SHUFFLE || !deflate_in || 
 	     deflate_level_in != deflate_level[i]) ERR;
       }
 
@@ -894,7 +919,7 @@ main(int argc, char **argv)
 	     ndims != 1 || natts != 0 || dimids_in[0] != 0) ERR;
 	 if (nc_inq_var_deflate(ncid, varid[i], &shuffle_in, &deflate_in, 
 				&deflate_level_in)) ERR;
-	 if (!shuffle_in || !deflate_in || 
+	 if (shuffle_in != NC_SHUFFLE || !deflate_in || 
 	     deflate_level_in != deflate_level[i]) ERR;
       }
 
@@ -922,7 +947,7 @@ main(int argc, char **argv)
       nc_type xtype_in;
       char name_in[NC_MAX_NAME + 1];
       int shuffle_in, deflate_in, deflate_level_in;
-      int fletcher32_in;
+      int checksum_in;
       int i;
 
       /* Create some phoney data. */
@@ -936,8 +961,8 @@ main(int argc, char **argv)
       {
 	 if (nc_def_var(ncid, var_name[i], NC_DOUBLE, NDIMS, dimids,
 			&varid[i])) ERR;
-	 if (nc_def_var_deflate(ncid, varid[i], 0, 1, 0)) ERR;
-	 if (nc_def_var_fletcher32(ncid, varid[i], 1)) ERR;
+	 if (nc_def_var_deflate(ncid, varid[i], NC_NOSHUFFLE, 1, 0)) ERR;
+	 if (nc_def_var_fletcher32(ncid, varid[i], NC_FLETCHER32)) ERR;
       }
 
       /* Check stuff. */
@@ -956,9 +981,9 @@ main(int argc, char **argv)
 	     ndims != 1 || natts != 0 || dimids_in[0] != 0) ERR;
 	 if (nc_inq_var_deflate(ncid, varid[i], &shuffle_in, &deflate_in, 
 				&deflate_level_in)) ERR;
-	 if (shuffle_in || !deflate_in || deflate_level_in != 0) ERR;
-	 if (nc_inq_var_fletcher32(ncid, varid[i], &fletcher32_in)) ERR;
-	 if (!fletcher32_in) ERR;
+	 if (shuffle_in != NC_NOSHUFFLE || !deflate_in || deflate_level_in != 0) ERR;
+	 if (nc_inq_var_fletcher32(ncid, varid[i], &checksum_in)) ERR;
+	 if (checksum_in != NC_FLETCHER32) ERR;
       }
 
       if (nc_close(ncid)) ERR;
@@ -980,10 +1005,10 @@ main(int argc, char **argv)
 	     ndims != 1 || natts != 0 || dimids_in[0] != 0) ERR;
 	 if (nc_inq_var_deflate(ncid, varid[i], &shuffle_in, &deflate_in, 
 				&deflate_level_in)) ERR;
-	 if (shuffle_in || !deflate_in || 
+	 if (shuffle_in != NC_NOSHUFFLE || !deflate_in || 
 	     deflate_level_in != 0) ERR;
-	 if (nc_inq_var_fletcher32(ncid, varid[i], &fletcher32_in)) ERR;
-	 if (!fletcher32_in) ERR;
+	 if (nc_inq_var_fletcher32(ncid, varid[i], &checksum_in)) ERR;
+	 if (checksum_in != NC_FLETCHER32) ERR;
       }
 
       if (nc_close(ncid)) ERR;
@@ -1003,7 +1028,7 @@ main(int argc, char **argv)
       nc_type xtype_in;
       char name_in[NC_MAX_NAME + 1];
       int shuffle_in, deflate_in, deflate_level_in;
-      int fletcher32_in, no_fill;
+      int checksum_in, no_fill;
       unsigned short ushort_data = 42, ushort_data_in, fill_value_in;
 
       /* Create a netcdf-4 file with one dim and 1 NC_USHORT var. */
@@ -1019,9 +1044,9 @@ main(int argc, char **argv)
 	  ndims != 1 || natts != 0 || dimids_in[0] != 0) ERR;
       if (nc_inq_var_deflate(ncid, 0, &shuffle_in, &deflate_in, 
 			     &deflate_level_in)) ERR;
-      if (shuffle_in || deflate_in) ERR;
-      if (nc_inq_var_fletcher32(ncid, 0, &fletcher32_in)) ERR;
-      if (fletcher32_in) ERR;
+      if (shuffle_in != NC_NOSHUFFLE || deflate_in) ERR;
+      if (nc_inq_var_fletcher32(ncid, 0, &checksum_in)) ERR;
+      if (checksum_in != NC_NOCHECKSUM) ERR;
       if (nc_inq_var_fill(ncid, 0, &no_fill, &fill_value_in)) ERR;
       if (no_fill || fill_value_in != NC_FILL_USHORT) ERR;
 
@@ -1047,9 +1072,9 @@ main(int argc, char **argv)
 	  ndims != 1 || natts != 0 || dimids_in[0] != 0) ERR;
       if (nc_inq_var_deflate(ncid, 0, &shuffle_in, &deflate_in, 
 			     &deflate_level_in)) ERR;
-      if (shuffle_in || deflate_in) ERR;
-      if (nc_inq_var_fletcher32(ncid, 0, &fletcher32_in)) ERR;
-      if (fletcher32_in) ERR;
+      if (shuffle_in != NC_NOSHUFFLE || deflate_in) ERR;
+      if (nc_inq_var_fletcher32(ncid, 0, &checksum_in)) ERR;
+      if (checksum_in != NC_NOCHECKSUM) ERR;
 
       if (nc_close(ncid)) ERR;
    }
@@ -1095,6 +1120,62 @@ main(int argc, char **argv)
       if (nc_close(ncid)) ERR;
    }
 
+   SUMMARIZE_ERR;
+   printf("*** testing fill values for 2D unlimited dimension variable...");
+   {
+#define D1_NAME "unlimited"
+#define D1_TARGET 3      
+#define D2_NAME "fixed"
+#define D2_LEN 3
+#define D2_TARGET 2
+#define V1_NAME "var1"
+#define ND1 2
+#define TARGET_VALUE 42      
+
+      int dimids[ND1];
+      size_t index[ND1];
+      int varid;
+      int no_fill;
+      int data = TARGET_VALUE, data_in[D1_TARGET][D2_LEN], fill_value_in;
+      int i, j;
+
+      /* Create a netcdf-4 file with one dim and 1 NC_USHORT var. */
+      if (nc_create(FILE_NAME, NC_NETCDF4, &ncid)) ERR;
+      if (nc_def_dim(ncid, D1_NAME, NC_UNLIMITED, &dimids[0])) ERR;
+      if (nc_def_dim(ncid, D2_NAME, D2_LEN, &dimids[1])) ERR;
+      if (nc_def_var(ncid, V1_NAME, NC_INT, ND1, dimids, &varid)) ERR;
+
+      /* Check stuff. */
+      if (nc_inq_var_fill(ncid, varid, &no_fill, &fill_value_in)) ERR;
+      if (no_fill) ERR;
+
+      /* Write a value. */
+      index[0] = D1_TARGET;
+      index[1] = D2_TARGET;
+      if (nc_put_var1_int(ncid, varid, index, &data)) ERR;
+
+      /* Get the data, and check the values. */
+      if (nc_get_var_int(ncid, 0, &data_in[0][0])) ERR;
+      for (i = 0; i < D1_TARGET; i++)
+	 for (j = 0; j < D2_LEN; j++)
+	    if ((i == D1_TARGET && j == D2_TARGET && data_in[i][j] != TARGET_VALUE) ||
+		data_in[i][j] != NC_FILL_INT) ERR;
+
+      if (nc_close(ncid)) ERR;
+
+      /* Open the file and check the same stuff. */
+      if (nc_open(FILE_NAME, NC_NOWRITE, &ncid)) ERR;
+
+      /* Get the data, and check the values. */
+      if (nc_get_var_int(ncid, 0, &data_in[0][0])) ERR;
+      for (i = 0; i < D1_TARGET; i++)
+	 for (j = 0; j < D2_LEN; j++)
+	    if ((i == D1_TARGET && j == D2_TARGET && data_in[i][j] != TARGET_VALUE) ||
+		data_in[i][j] != NC_FILL_INT) ERR;
+
+      if (nc_close(ncid)) ERR;
+   }
+
    SUMMARIZE_ERR;
    printf("*** testing lots of variables...");
 #define DIM_A_NAME "x"
@@ -1116,38 +1197,41 @@ main(int argc, char **argv)
       for (i = 0; i < NUM_VARS; i++)
       {
 	 sprintf(varname, "a_%d", i);
-	 if (nc_def_var(ncid, varname, NC_FLOAT, 1, dimids, &varids[i])) ERR;
+	 if (nc_def_var(ncid, varname, NC_FLOAT, 1, dimids, &varids[i])) {
+	    ERR;
+	    break;
+	 }
       }
       if (nc_close(ncid)) ERR;
    }
-
    SUMMARIZE_ERR;
+
 #define NC3_CLASSIC_FILE "tst_pres_temp_4D_classic.nc"
 #define NC3_64BIT_OFFSET_FILE "tst_pres_temp_4D_64bit_offset.nc"
 #define NC3_NETCDF4_FILE "tst_pres_temp_4D_netcdf4.nc"
 #define NC3_NETCDF4_CLASSIC_FILE "tst_pres_temp_4D_netcdf4_classic.nc"
+
    printf("*** testing 4D example file in classic format...");
    if (create_4D_example(NC3_CLASSIC_FILE, NC_CLOBBER)) ERR;
    if (check_4D_example(NC3_CLASSIC_FILE, NC_FORMAT_CLASSIC)) ERR;      
    SUMMARIZE_ERR;
+
    printf("*** testing 4D example file in 64-bit offset format...");
    if (create_4D_example(NC3_64BIT_OFFSET_FILE, NC_CLOBBER|NC_64BIT_OFFSET)) ERR;
    if (check_4D_example(NC3_64BIT_OFFSET_FILE, NC_FORMAT_64BIT)) ERR;      
    SUMMARIZE_ERR;
+
    printf("*** testing 4D example file in netCDF-4/HDF5 format...");
    if (create_4D_example(NC3_NETCDF4_FILE, NC_CLOBBER|NC_NETCDF4)) ERR;
    if (check_4D_example(NC3_NETCDF4_FILE, NC_FORMAT_NETCDF4)) ERR;      
    SUMMARIZE_ERR;
+
    printf("*** testing 4D example file in netCDF-4/HDF5 format with classic model rules...");
    if (create_4D_example(NC3_NETCDF4_CLASSIC_FILE, NC_CLOBBER|NC_NETCDF4|NC_CLASSIC_MODEL)) ERR;
    if (check_4D_example(NC3_NETCDF4_CLASSIC_FILE, NC_FORMAT_NETCDF4_CLASSIC)) ERR;      
    SUMMARIZE_ERR;
 
    FINAL_RESULTS;
-#ifdef USE_PARALLEL
-   MPI_Finalize();
-#endif   
-
 }
 
 
