@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/ncdump/vardata.c,v 1.39 2009/02/14 20:44:16 russ Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/ncdump/vardata.c,v 1.47 2010/02/04 16:49:13 russ Exp $
  *********************************************************************/
 
 #include <config.h>
@@ -73,7 +73,7 @@ print_any_att_val (
     const void *valp		/* pointer to the value */
 	    ) {
     nctype_t *typ = attp->tinfo;
-    (void *) (*typ->typ_tostring)(typ, sb, valp);
+    (*typ->typ_tostring)(typ, sb, valp);
 }
 
 
@@ -95,7 +95,7 @@ print_any_val(
 				     (const void*)varp->fillvalp, valp) ) {
 	sbuf_cpy(sb, FILL_STRING);
     } else {
-	(void *) (*varp->val_tostring)(varp, sb, valp);
+	(*varp->val_tostring)(varp, sb, valp);
     }
 }
 
@@ -226,8 +226,7 @@ pr_any_vals(
 	print_any_val(sb, vp, (void *)valp);
 	valp += vp->tinfo->size; /* next value according to type */
 	if (fsp->full_data_cmnts) {
-	    printf(sb->buf);
-	    printf(",");
+	    printf("%s, ", sb->buf);
 	    annotate (vp, fsp, cor, iel);
 	} else {
 	    sbuf_cat(sb, ", ");
@@ -235,9 +234,8 @@ pr_any_vals(
 	}
     }
     print_any_val(sb, vp, (void *)valp);
-    valp += vp->tinfo->size;	/* next value according to type */
     if (fsp->full_data_cmnts) {
-	printf(sbuf_str(sb));
+	printf("%s", sbuf_str(sb));
 	lastdelim (more, lastrow);
 	annotate (vp, fsp, cor, iel);
     } else {
@@ -428,7 +426,7 @@ vardata(
 		&& left > 0) {	/* print brief comment with indices range */
 /* 		printf("// %s(",vp->name); */
 		printf("// ");
-		printf(vp->name);
+		print_name(vp->name);
 		printf("(");
 		switch (fsp->data_lang) {
 		  case LANG_C:
@@ -462,7 +460,7 @@ vardata(
 	    size_t toget = left < gulp ? left : gulp;
 	    if (vrank > 0)
 	      edg[vrank-1] = toget;
-	    NC_CHECK(nc_get_vara(ncid, varid, cor, edg, vals) );
+	    NC_CHECK(nc_get_vara(ncid, varid, cor, edg, vals));
 	    /* Test if we should treat array of chars as a string  */
 	    if(vp->type == NC_CHAR && 
 	       (vp->fmt == 0 || STREQ(vp->fmt,"%s") || STREQ(vp->fmt,""))) {
@@ -599,7 +597,6 @@ pr_any_valsx(
 	lput(sbuf_str(sb));
     }
     print_any_val(sb, vp, (void *)valp);
-    valp += vp->tinfo->size;
     lput(sbuf_str(sb));
     lastdelim2x (more, lastrow);
     sbuf_free(sb);
