@@ -1,13 +1,15 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf/nctest/cdftests.c,v 1.14 1996/02/07 18:06:05 steve Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/nctest/cdftests.c,v 1.23 2009/02/14 14:11:28 ed Exp $
  *********************************************************************/
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>		/* for free() */
 #include "netcdf.h"
+#include "emalloc.h"
 #include "testcdf.h"		/* defines in-memory test netcdf structure */
 #include "add.h"		/* functions to update in-memory netcdf */
 #include "error.h"
@@ -24,9 +26,9 @@
  * On exit, netcdf files are closed.
  * Uses: nccreate, ncendef, ncclose, ncopen.
  */
-void
+int
 test_nccreate(path)
-     char *path;		/* name of netCDF file to create */
+     const char *path;		/* name of netCDF file to create */
 {
     int nerrs = 0;
     static char pname[] = "test_nccreate";
@@ -36,7 +38,7 @@ test_nccreate(path)
 
     if ((ncid = nccreate(path, NC_CLOBBER)) == -1) {
 	error("%s: nccreate failed to NC_CLOBBER", pname);
-	return;
+	return 1;
     }
     /* in define mode already, so ncredef should fail  */
     if (ncredef(ncid) != -1) {
@@ -54,7 +56,7 @@ test_nccreate(path)
     }
     if ((ncid = ncopen(path, NC_NOWRITE)) == -1) {
 	error("%s: ncopen of newly created netcdf failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened OK */
     if (ncclose (ncid) == -1) {
@@ -73,6 +75,8 @@ test_nccreate(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -86,9 +90,11 @@ test_nccreate(path)
  * On exit, netcdf files are closed.
  * Uses: ncopen, ncredef, ncattput, ncendef, ncclose.
  */
-void
+#define DATA_LEN 32    
+#define TEMP_FILE_NAME "temp.tmp"
+int
 test_ncopen(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncopen";
@@ -97,6 +103,8 @@ test_ncopen(path)
     static char xpath[] = "tooth-fairy.nc"; /* must not exist */
     static struct cdfatt title = /* attribute */
       {NC_GLOBAL, "title", NC_CHAR, LEN_OF (title_val), (void *) title_val};
+    FILE *temp;
+    char dummy_data[DATA_LEN];
 
     (void) fprintf(stderr, "*** Testing %s ...\t\t", &pname[5]);
 
@@ -104,7 +112,7 @@ test_ncopen(path)
     if((ncid0 = ncopen(xpath, NC_NOWRITE)) != -1) {
 	error("%s: ncopen should fail opening nonexistent file",
 	      pname);
-	return;
+	return ++nerrs;
     }
     if (ncerr != NC_SYSERR) {
 	error("%s: ncopen of nonexistent file should set ncerr to %d",
@@ -112,58 +120,72 @@ test_ncopen(path)
     }
     /*
      * Open a non-netCDF file.  Don't use "Makefile.in" because that
-     * name is munged to something else by PC/NFS and, consequently, won't
-     * exist in a cross-mounted directory.
+     * name is munged to something else by PC/NFS and, consequently,
+     * won't exist in a cross-mounted directory. Also don't use a
+     * source file, becase that will break building in another
+     * directory, and consequently, make dist. An object file is not
+     * safe, because sometimes it's a .o and sometimes a .obj. So just
+     * create a file!
      */
-    if((ncid0 = ncopen("driver.c", NC_NOWRITE)) != -1) {
+    if (!(temp = fopen(TEMP_FILE_NAME, "w+")))
+    {
+       error("could not create temp file");
+       return ++nerrs;
+    }
+    if (fwrite(dummy_data, 1, DATA_LEN, temp) != DATA_LEN)
+    {
+       error("could not write to temp file");
+       return ++nerrs;
+    }
+    if (fclose(temp))
+    {
+       error("could not close temp file");
+       return ++nerrs;
+    }
+    
+    if((ncid0 = ncopen(TEMP_FILE_NAME, NC_NOWRITE)) != -1) {
 	error("%s: ncopen should fail opening non-netCDF file",
 	      pname);
-	return;
+	return ++nerrs;
     }
     if(ncerr != NC_ENOTNC) {
 	error("%s: ncopen of non-netCDF file should set ncerr to %d",
 	      pname, NC_ENOTNC);
-	return;
+	return ++nerrs;
     }
 
 
     if ((ncid0 = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed with NC_WRITE mode", pname);
-	return;
+	return ++nerrs;
     }
 
     /* opened */
     if (ncredef(ncid0) == -1) {
 	error("%s: cdredef failed", pname);
-	ncclose(ncid0); return;
+	ncclose(ncid0); return ++nerrs;
     }
     /* in define mode */
     if (ncattput(ncid0, NC_GLOBAL, "title", NC_CHAR, title.len, title.val)
 	== -1) {
 	error("%s: ncattput failed", pname);
-	ncclose(ncid0); return;
+	ncclose(ncid0); return ++nerrs;
     }
     add_att(&test, NC_GLOBAL, &title); /* keep in-memory netcdf updated */
     if (ncendef (ncid0) == -1) {
 	error("%s: ncendef failed after ncattput", pname);
-	ncclose(ncid0); return;
+	ncclose(ncid0); return ++nerrs;
     }
     if (ncclose (ncid0) == -1) {
 	error("%s: ncclose failed in NC_WRITE mode", pname);
-	return;
+	return ++nerrs;
     }
 
-    if ((ncid0 = ncopen(path, NC_NOWRITE)) == -1) {
+    if ((ncid0 = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed with NC_NOWRITE mode", pname);
-	return;
+	return ++nerrs;
     }
-    /* opened */
-    /* this should fail, since in NC_NOWRITE mode */
-    if (ncredef(ncid0) != -1) {
-	error("%s: cdredef should fail after NC_NOWRITE open", pname);
-	ncclose(ncid0); return;
-    }
-    if ((ncid1 = ncopen(path, NC_WRITE)) == -1) {
+    if ((ncid1 = ncopen(path, NC_NOWRITE)) == -1) {
 #ifndef vms
 	error("%s: second ncopen failed", pname);
 	nerrs++;
@@ -173,16 +195,21 @@ test_ncopen(path)
     }
     else
     {
-	    /* second open OK */
-	    if (ncid0 == ncid1) {
-		error("%s: ncopen should return new ncid on second open",
-		      pname);
-		nerrs++;
-	    }
-	    if (ncclose (ncid1) == -1) {
-		error("%s: ncclose failed to close after second open", pname);
-		nerrs++;
-	    }
+       /* this should fail, since in NC_NOWRITE mode */
+       if (ncredef(ncid1) != -1) {
+	  error("%s: cdredef should fail after NC_NOWRITE open", pname);
+	  ncclose(ncid1); return ++nerrs;
+       }
+       /* second open OK */
+       if (ncid0 == ncid1) {
+	  error("%s: ncopen should return new ncid on second open",
+		pname);
+	  nerrs++;
+       }
+       if (ncclose (ncid1) == -1) {
+	  error("%s: ncclose failed to close after second open", pname);
+	  nerrs++;
+       }
     }
     if (ncclose (ncid0) == -1) {
 	error("%s: ncclose failed in NC_NOWRITE mode", pname);
@@ -192,6 +219,8 @@ test_ncopen(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -204,9 +233,9 @@ test_ncopen(path)
  * On exit netcdf files are closed.
  * Uses: ncopen, ncredef, ncdimdef, ncvardef, ncattput, ncclose 
  */
-void
+int
 test_ncredef(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncredef";
@@ -225,17 +254,17 @@ test_ncredef(path)
 
     if ((ncid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened OK, enter define mode */
     if (ncredef(ncid) == -1) {
 	error("%s: cdredef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     /* in define mode OK, add a dimension */
     if ((ii_dim = ncdimdef(ncid, ii.name, ii.size)) == -1) {
 	error("%s: ncdimdef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     add_dim(&test, &ii);	/* keep in-memory netcdf in sync */
 
@@ -245,7 +274,7 @@ test_ncredef(path)
     if ((aa_id = ncvardef(ncid, aa.name, aa.type,
 			   aa.ndims, aa.dims)) == -1) {
 	error("%s: ncvardef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     add_var(&test, &aa);	/* keep in-memory netcdf in sync */
 
@@ -254,31 +283,33 @@ test_ncredef(path)
     if (ncattput(ncid, aa_units.var, aa_units.name,
 		  aa_units.type, aa_units.len, (void *) aa_units.val) == -1) {
 	error("%s: ncattput failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     add_att(&test, aa_id, &aa_units); /* keep in-memory netcdf in sync */
 
     if (ncredef(ncid) != -1) {
 	error("%s: cdredef in define mode should have failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     if (ncendef (ncid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     if (ncclose (ncid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     if (ncredef(ncid) != -1) {
 	error("%s: ncredef failed to report bad netcdf handle", pname);
 	nerrs++;
     }
-    free ((char *)aa.dims);
+    free (aa.dims);
     if (nerrs > 0)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -290,9 +321,9 @@ test_ncredef(path)
  *  On exit netcdf files are closed.
  * Uses: ncopen, ncredef, ncdimdef, ncvardef, ncattput, ncendef, ncclose
  */
-void
+int
 test_ncendef(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncendef";
@@ -313,18 +344,18 @@ test_ncendef(path)
 
     if ((ncid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened */
     if (ncredef(ncid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     /* in define mode, add dimensions */
     if ((jj_dim = ncdimdef(ncid, jj.name, jj.size)) == -1 ||
 	(kk_dim = ncdimdef(ncid, kk.name, kk.size)) == -1) {
 	error("%s: ncdimdef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     add_dim(&test, &jj);	/* keep in-memory netcdf in sync */
     add_dim(&test, &kk);	/* keep in-memory netcdf in sync */
@@ -336,7 +367,7 @@ test_ncendef(path)
     if ((bb_id = ncvardef(ncid, bb.name, bb.type,
 			   bb.ndims, bb.dims)) == -1) {
 	error("%s: ncvardef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     add_var(&test, &bb);	/* keep in-memory netcdf in sync */
     
@@ -344,22 +375,22 @@ test_ncendef(path)
     if (ncattput(ncid, bb_id, bb_range.name, bb_range.type, bb_range.len,
 		  (void *) bb_range.val) == -1) {
 	error("%s: ncattput failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     add_att(&test, bb_id, &bb_range); /* keep in-memory netcdf in sync */
     
     if (ncendef (ncid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     /* in data mode */
     if (ncendef (ncid) != -1) { /* should fail in data mode */
 	error("%s: ncendef in data mode should have failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     if (ncclose (ncid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* should fail on a bad handle */
     if (ncendef (ncid) != -1) {
@@ -370,6 +401,8 @@ test_ncendef(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -380,9 +413,9 @@ test_ncendef(path)
  *    try with bad handle, check error
  *  On exit netcdf files are closed.
  */
-void
+int
 test_ncclose(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncclose";
@@ -392,12 +425,12 @@ test_ncclose(path)
 
     if ((ncid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened */
     if (ncredef(ncid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     /* in define mode */
     if (ncclose (ncid) == -1) {
@@ -407,7 +440,7 @@ test_ncclose(path)
 
     if ((ncid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* in data mode */
     if (ncclose (ncid) == -1) {
@@ -422,6 +455,8 @@ test_ncclose(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -432,9 +467,9 @@ test_ncclose(path)
  *    try with bad handle, check error
  *  On exit netcdf files are closed.
  */
-void
+int
 test_ncinquire(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncinquire";
@@ -467,12 +502,12 @@ test_ncinquire(path)
 
     if ((ncid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened, in data mode */
     if (ncinquire(ncid, &ndims, &nvars, &ngatts, &xdimid) == -1) {
 	error("%s: ncinquire in data mode failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     /* compare returned with expected values */
     if (ndims != test.ndims) {
@@ -498,14 +533,14 @@ test_ncinquire(path)
 
     if (ncredef(ncid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     /* add dimensions */
     for (id = 0; id < nd; id++) {
 	if ((dimids[id] = ncdimdef(ncid, dims[id].name, dims[id].size))
 	    == -1) {
 	    error("%s: ncdimdef failed on normal dimension", pname);
-	    ncclose(ncid); return;
+	    ncclose(ncid); return ++nerrs;
 	}
 	add_dim(&test, &dims[id]);
     }
@@ -513,7 +548,7 @@ test_ncinquire(path)
     /* add an unlimited dimension */
     if ((rec_dim = ncdimdef(ncid, rec.name, rec.size)) == -1) {
 	error("%s: ncdimdef failed on NC_UNLIMITED dimension", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     add_dim(&test, &rec);
 
@@ -526,7 +561,7 @@ test_ncinquire(path)
 	if ((cc_id = ncvardef(ncid, cc[iv].name, cc[iv].type,
 			       cc[iv].ndims, cc[iv].dims)) == -1) {
 	    error("%s: ncvardef failed", pname);
-	    ncclose(ncid); return;
+	    ncclose(ncid); return ++nerrs;
 	}
 	add_var(&test, &cc[iv]);
 
@@ -534,14 +569,14 @@ test_ncinquire(path)
 	if (ncattput(ncid, cc_id, cc_units.name, cc_units.type,
 		      cc_units.len, (void *) cc_units.val) == -1) {
 	    error("%s: ncattput failed", pname);
-	    ncclose(ncid); return;
+	    ncclose(ncid); return ++nerrs;
 	}
 	add_att(&test, cc_id, &cc_units);
     }
     /* try calling from define mode, compare returned values to expected */
     if (ncinquire(ncid, &ndims, &nvars, &ngatts, &xdimid) == -1) {
 	error("%s: ncinquire in define mode failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     /* compare returned with expected values */
     if (ndims != test.ndims) {
@@ -567,12 +602,12 @@ test_ncinquire(path)
 
     if (ncendef (ncid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
 
     if (ncclose (ncid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* should fail, since bad handle */
     if (ncinquire (ncid, &ndims, &nvars, &ngatts, &xdimid) != -1) {
@@ -583,6 +618,8 @@ test_ncinquire(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -593,9 +630,9 @@ test_ncinquire(path)
  *    try with bad handle, check error
  *  On exit netcdf files are closed.
  */
-void
+int
 test_ncsync(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncsync";
@@ -614,19 +651,19 @@ test_ncsync(path)
 
     if ((ncid0 = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen in NC_WRITE mode failed", pname);
-	return;
+	return ++nerrs;
     }
 
     /* opened */
     if (ncredef(ncid0) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(ncid0); return;
+	ncclose(ncid0); return ++nerrs;
     }
     /* in define mode, add a dimension, variable, and attribute */
     if ((ll_dim = ncdimdef(ncid0, ll.name, ll.size)) == -1) {
 	error("%s: ncdimdef failed", pname);
 	ncclose(ncid0);
-	return;
+	return ++nerrs;
     }
     add_dim(&test, &ll);
 
@@ -635,7 +672,7 @@ test_ncsync(path)
     if ((dd_id=ncvardef(ncid0, dd.name, dd.type, dd.ndims, dd.dims)) == -1) {
 	error("%s: ncvardef failed", pname);
 	ncclose(ncid0);
-	return;
+	return ++nerrs;
     }
     add_var(&test, &dd);
 
@@ -643,7 +680,7 @@ test_ncsync(path)
 		  dd_fill_val.len, (void *) dd_fill_val.val) == -1) {
 	error("%s: ncattput failed", pname);
 	ncclose(ncid0);
-	return;
+	return ++nerrs;
     }
     add_att(&test, dd_id, &dd_fill_val);
 
@@ -654,7 +691,7 @@ test_ncsync(path)
 
     if (ncendef (ncid0) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(ncid0); return;
+	ncclose(ncid0); return ++nerrs;
     }
     /* in data mode */
     if (ncsync (ncid0) == -1) {
@@ -672,7 +709,7 @@ test_ncsync(path)
 	if (ncvarput(ncid0,dd_id,dd_start,dd_edges,(void *)dd_vals) == -1) {
 	    error("%s: ncvarput failed", pname);
 	    ncclose(ncid0);
-	    return;
+	    return ++nerrs;
 	}
 	add_data(&test,dd_id,dd_start,dd_edges); /* keep test in sync */
 	if (ncsync (ncid0) == -1) {
@@ -719,6 +756,8 @@ test_ncsync(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -729,9 +768,9 @@ test_ncsync(path)
  *    try with bad handle, check error
  *  On exit netcdf files are closed.
  */
-void
+int
 test_ncabort(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncabort";
@@ -745,51 +784,51 @@ test_ncabort(path)
 
     if ((ncid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened */
     if (ncredef(ncid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     /* in define mode, add a new global attribute */
     if (ncattput(ncid, NC_GLOBAL, att.name, att.type, att.len, att.val) == -1) {
 	error("%s: ncattput failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
 
     /* abort in define mode, should restore to state before define mode */
     if (ncabort(ncid) == -1) {
 	error("%s: ncabort in define mode failed", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     if ((ncid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen after ncabort failed", pname);
-	return;
+	return ++nerrs;
     }
     /* check that new global attribute was not added */
     if (ncattinq(ncid, NC_GLOBAL, att.name, &att.type, &att.len) != -1) {
 	error("%s: ncabort should have restored state before ncredef", pname);
-	ncclose(ncid); return;
+	ncclose(ncid); return ++nerrs;
     }
     /* in data mode not being created, should just close */
     if (ncabort(ncid) == -1) {
 	error("%s: ncabort in define mode failed", pname);
-	return;
+	return ++nerrs;
     }
     if ((ncid = nccreate(fpath, NC_CLOBBER)) == -1) {
 	error("%s: nccreate failed to NC_CLOBBER", pname);
-	return;
+	return ++nerrs;
     }
     /* in define mode being created, should delete */
     if (ncabort(ncid) == -1) {
 	error("%s: ncabort after nccreate failed", pname);
-	return;
+	return ++nerrs;
     }
     /* check with ncopen that file doesn't exist */
     if (ncopen(fpath, NC_NOWRITE) != -1) {
 	error("%s: ncabort deleted file, but ncopen found it", pname);
-	return;
+	return ++nerrs;
     }
     if (ncabort(ncid) != -1) {	/* should fail, ncid is bad handle */
 	error("%s: ncclose failed to report bad netcdf handle", pname);
@@ -799,4 +838,6 @@ test_ncabort(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
